'''
Created on May 14, 2012

@author: pascale
'''
# -*- coding: utf-8 -*-
#wxPython 2.8
try :
    import wx
    if wx.VERSION[1]== 8 :
        import sys 
        sys.stderr.write(' wx python version = 8  some features of RTTOVGUI might not work properly\n')

except :
    import sys 
    sys.stderr.write('ERROR: wx is not installed or not compatible with python \n')
    sys.exit(1)

if wx.VERSION[1]== 8 :
    from wx.lib.pubsub import Publisher as pub
else    :    
# wxPython 2.9 :
#wx.lib.pubsub: Pusub now defaults to the new "kwarg" version of the API. 
#In order to continue using the original "arg1" API you
# will need to import wx.lib.pubsub.setuparg1 before importing any other pubsub modules.
    import wx.lib.pubsub.setuparg1
    from wx.lib.pubsub import pub
    
    
try :
    import   datetime
except ImportError:
    import sys 
    sys.stderr.write('ERROR: datetime is not installed\n')
    sys.exit(1)
    
try :
    import h5py
except ImportError:
    import sys 
    sys.stderr.write('ERROR: h5py  is not installed\n')
    sys.exit(1)
 
import rttov

import config
import sys
import os
import rttov_gui_f2py
import logging
import numpy
import logging
import signal


  
class Coefficients(object):
    def __init__(self):
        self.loadCoeffs=False
        self.WAVENUMBERS=None
        self.fileName={"standard":"","aerosols":"","clouds":"","PC":""}
        self.nchannels=0
        # gas_id : cf rttov_const.F90        
#        & gas_id_mixed       = 1, &
#        & gas_id_watervapour = 2, &
#        & gas_id_ozone       = 3, &
#        & gas_id_wvcont      = 4, &
#        & gas_id_co2         = 5, &
#        & gas_id_n2o         = 6, &
#        & gas_id_co          = 7, &
#        & gas_id_ch4         = 8
        self.gas_id={'Q':1,'O3':3,'CO2':5,"N2O":6,"CO":7,"CH4":8} 
    def isMW(self):
        """ return True if the sensor is a MicroWave sensor """
        if self.loadCoeffs :
            ID_SENSOR=rttov.getcoefval.rttov_get_coef_val_i0("ID_SENSOR")
            if ID_SENSOR == 2 or ID_SENSOR == 4 :
                return True
            else :
                return False
        else :
            return False     
        
    def hasSolar(self):
        if self.loadCoeffs :
            SS_VAL_CHN=rttov.getcoefval.rttov_get_coef_val_i1("SS_VAL_CHN")
            if SS_VAL_CHN.sum() > 0 :
                return True
            else :
                return False
        else :
            return False         
        
    def hasGas(self,gas):
        if self.loadCoeffs :
            if rttov.getcoefval.rttov_get_coef_val_i1('FMV_GAS_POS')[self.gas_id[gas]-1] == 0 :
                return False
            else :
                return True
        else :
            return False    
    def getFMV_GAS_POS  (self) :
        if self.loadCoeffs :
            return rttov.getcoefval.rttov_get_coef_val_i1('FMV_GAS_POS')
        else :
            return None
    
    def hasPC(self):
        if (self.loadCoeffs) and self.fileName["PC"] != "" :
            return True
        else:
            return False    
    def hasClouds(self):
        if (self.loadCoeffs) and self.fileName["clouds"] != "" :
            return True
        else:
            return False   
  
    def hasAerosols(self):
        if (self.loadCoeffs) and self.fileName["aerosols"] != "" :
            return True
        else:
            return False   
   
    def isPCClouds(self): 
        if self.loadCoeffs and self.fileName["PC"] != "" :
            cloud=rttov.getcoefval.rttov_get_coef_val_i0('FMV_PC_CLD')
            if cloud !=  0  and self.hasClouds() :
                return True
            else :
                return False
        else :
            return False
    
    def getFMV_PC_BANDS(self):
        if  self.loadCoeffs :
            if self.hasPC() :
                return rttov.getcoefval.rttov_get_coef_val_i0('FMV_PC_BANDS')
        else :
            return None    
    
    def getFMV_PC_SETS(self):
        if  self.loadCoeffs :
            if self.hasPC() :
                return rttov.getcoefval.rttov_get_coef_val_i1('FMV_PC_SETS')   
        else :
            return None    
    
    def isHighResolution(self):
        """ return True if the sensor is a Hyper spectral sensor """
        if self.loadCoeffs :
            if  rttov.getcoefval.rttov_get_coef_val_i0('ID_SENSOR') == 3 :
                return True
            else   : 
                return False
        else :
            return False
            
    
    def hasNLTE(self):
        """ return True if the sensor can do an NLTE correction """

        if self.loadCoeffs :
            nlte=rttov.getcoefval.rttov_get_coef_val_i0("NLTECOEF")
            if  nlte == 1 :
                return True
            else   : 
                return False
        else :
            return False
       
        
class pProfile(rttov.profile.Profile):
    """ project Profile Class : add specific functionalities to the rttov Profile Class""" 
    def __init__(self):
        rttov.profile.Profile.__init__(self)
        
    def ctrlCoherenceClouds(self):
        """ control the cloud coherence """
        cloudCumulDensity=numpy.zeros( self['NLAYERS'], dtype=self['P'].dtype )
        if self.anyCloud()    :      
            for cloud in self.cloud_list :
                if self[cloud] != None :
                    cloudCumulDensity=cloudCumulDensity+self[cloud]
                    
            for layer in range(self['NLAYERS']):
                if cloudCumulDensity[layer]  == 0 :
                    self['CFRAC'][layer] = 0.
                else :
                    if self['CFRAC'][layer] == 0. :
                        self['CFRAC'][layer]=   0.20
    def hasGas(self,gas) :
        if self.has_key(gas) :
            if self[gas] != None :
                return True
            else :
                return False
        else :
            return True
 
    def hasClouds(self) :
        hasclouds=False    
        for cloud in self.cloud_list :
            if self.has_key(cloud) :
                if self[cloud] != None : 
                    hasclouds=True
        return hasclouds   
    def hasAerosols(self) :
        hasAer=False    
        for aer in self.aerosol_list :
            if self.has_key(aer) :
                if self[aer] != None : 
                    hasAer=True
        return hasAer    
          
class pOption(rttov.option.Option):
    """ project Option Class : add specific functionalities to the rttov Option Class """ 
    optionsThemes=['General configuration options','VIS/IR-only RT options','General RT options','PC-RTTOV options','MW-only RT options','Interpolation and vertical grid options']   
    optionsThemesListe={ "General configuration options": ("APPLY_REG_LIMITS","VERBOSE","DO_CHECKINPUT" ),
                         "General RT options" : ("ADDREFRAC","SWITCHRAD","USE_Q2M"),
                         "VIS/IR-only RT options" : ("ADDSOLAR","DO_NLTE_CORRECTION","ADDAEROSL","ADDCLOUDS",
                                                     "CLDSTR_THRESHOLD","OZONE_DATA","CO2_DATA","N2O_DATA","CO_DATA",
                                                     "CH4_DATA","USER_AER_OPT_PARAM","USER_CLD_OPT_PARAM","CLDSTR_SIMPLE"),
                        "PC-RTTOV options" :("ADDPC","IPCBND","IPCREG","ADDRADREC"),
                        "MW-only RT options":("FASTEM_VERSION","CLW_DATA","DO_LAMBERTIAN"),
                        "Interpolation and vertical grid options" : ("ADDINTERP","INTERP_MODE","SPACETOP","LGRADP","REG_LIMIT_EXTRAP")
                        }

    #3 possibilittes:
    #Integer(Kind=jpim), Parameter :: ninterp_modes           = 3_jpim ! Number of valid interpolation options
    #Integer(Kind=jpim), Parameter :: interp_rochon           = 1_jpim ! Rochon interpolation everywhere
 #  Integer(Kind=jpim), Parameter :: interp_loglinear        = 2_jpim ! Log-linear interpolation everywhere
#Integer(Kind=jpim), Parameter :: interp_rochon_loglinear = 3_jpim ! Rochon for user->coef, log-lin. for coef->user

    
    
    fullIPCREG={1:"1",2:'2',3:'3',4:'4'}
    fullIPCBND={1:"full spectrum",2:'longwave subset',3:'short wave subset'}
    comboChoice={"IPCREG":fullIPCREG
               , 'IPCBND':fullIPCBND
               , 'FASTEM_VERSION':{1:"1",2:'2',3:'3',4:'4',5:'5',6:'6'}
               ,'INTERP_MODE':{1:"1 Rochon/Rochon OD",2:'2 Log-Lin/Log-lin OD',3:'3 Rochon/Log-lin OD',
                               4:'4 Rochon/Rochon WF',5:'5 Rochon/Log-lin WF'}
                }
    
    statusOption={'ADDAEROSL':False, 'ADDCLOUDS':False, 'ADDINTERP':False, 'ADDPC':False, 'ADDRADREC':False, 
              'ADDREFRAC':False, 'ADDSOLAR':False, 'APPLY_REG_LIMITS':False,"CH4_DATA":False, 
              'CLW_DATA':False, 'CO2_DATA':False, 'CO_DATA':False, 'DO_CHECKINPUT':False, 'DO_LAMBERTIAN':False, 
              'LGRADP':False, 'N2O_DATA':False, 'OZONE_DATA':False, 
              'SPACETOP':False, 'SWITCHRAD':False, 'USE_Q2M':False, 'VERBOSE':False ,
              'USER_AER_OPT_PARAM':False, 'USER_CLD_OPT_PARAM':False, 'USE_Q2M':False, 'DO_NLTE_CORRECTION':False, 
              'CLDSTR_THRESHOLD':False, 'CLDSTR_SIMPLE':False,'FASTEM_VERSION':False, 'IPCREG':False, 'IPCBND':False, 'INTERP_MODE':False ,'REG_LIMIT_EXTRAP':True}    
                  
    namesInProfile={'CO_DATA':"CO",'OZONE_DATA':"O3",'N2O_DATA':"N2O",'CH4_DATA':"CH4","CO2_DATA":"CO2"
                     }     
    # correspondance between a profile item and a option naime        
    profileItemOptionName={"CO":'CO_DATA',"O3":'OZONE_DATA',"N2O":'N2O_DATA',"CH4":'CH4_DATA',"CO2":"CO2_DATA","CLW":"CLW_DATA"}   
    
    ipcregNbPredictors={1: {"IASI": {1:300,2:400,3:500,4:600},
                            "AIRS": {1:200,2:300,3:400} },
                        2 : {"IASI":{1:35,2:40,3:45,4:50}},
                        3 : {"IASI": {1:30,2:40,3:50,4:60}}
                        }             
    def __init__(self):
        rttov.option.Option.__init__(self)
        # RTTOV options structure
        
    def updatePCChoicesList(self,myCoeff):
        """ update the PCChoicesList according to coefficients files"""
        """ FMV_PC_BAND = return Bands availables in the coefficient file """
        """ FMV_PC_SET = return max ipcreg  for each band """
   
        FMV_PC_BAND=myCoeff.getFMV_PC_BANDS()
        FMV_PC_SET=myCoeff.getFMV_PC_SETS()
        self.comboChoice["IPCBND"]={}
        for band in range(1,FMV_PC_BAND+1):
            self.comboChoice["IPCBND"][band]=self.fullIPCBND[band]
        maxIPCREG=self.getMaxIPCREG(myCoeff,self["IPCBND"])    
        self.comboChoice["IPCREG"]={}
        for val in range(1,maxIPCREG+1):
            self.comboChoice["IPCREG"][val]=self.fullIPCREG[val]

        
    def getMaxIPCREG(self,myCoeff,band):

        FMV_PC_SET=myCoeff.getFMV_PC_SETS()

        maxIPCREG=FMV_PC_SET[band-1]
        #print "maxIPCREG",maxIPCREG
#        self.comboChoice["IPCREG"]={}
#        for val in range(1,maxIPCREG+1):
#            self.comboChoice["IPCREG"][val-1]=self.fullIPCBND[val]
        return maxIPCREG
           
            
    
class Project(object):
    '''
    The Project class makes the interface with rttov_hdf_mod and rttov_guy_f2py 
    attributes :
    myCoeffs : to store the name of the coefficients files and to know if they are loaded or not
    myProfile : a rttov_hdf_mod Profile object 
    myOption :  a rttov_hdf_mod Option object
    config : configuration (path ans so on)
    
    '''
    # take care number of threads should be 1 if the instrument contains
    # both pure solar and pure thermal channels
    #   if the add_solar option is false a thread can have only pure solar channels
    #   so this thred fails with message "No thermal or solar calculations to carry out"
    nthreads=1
    def __init__(self):
        '''
        Constructor
        '''
        
        self.debug=False
        self.myCoeffs=Coefficients()
        self.myProfile=pProfile()
        self.myOption=pOption()
        self.myEmissivity=rttov.emissivity.Emissivity()
        self.myReflectance=rttov.reflectance.Reflectance()
        self.loadProfile=False
        self.useAtlas=False
        self.atlasLon=0
        self.atlasLat=0
        self.firstTimeSurface=True
        self.config=config.Config()
        # name files configured by user
        self.savedProfileFileName=self.config.profileDefaultFileName
        self.savedProjectFileName=None
        self.savedSurfaceFileName=None
        # default fileName
        self.profileFileName=self.config.profileDefaultFileName
        self.radianceFileName=self.config.radianceDefaultFileName
        self.KMatrixFileName=self.config.KMatrixDefaultFileName
        self.transFileName=self.config.transDefaultFileName
        self.surfaceFileName=self.config.surfaceDefaultFileName
        self.pcFileName=self.config.pcDefaultFileName
        self.pcKMatrixFileName=self.config.pcKMatrixFileName
        self.RTTOV_MODE="DIRECT"
        self.RTTOV_MODE_K="K"
        # create logger
        self.myLogger = logging.getLogger('rttov gui')
        formatter = logging.Formatter('%(asctime)s %(levelname)s %(message)s')
        self.ListeParamAtlas=["LATITUDE","LONGITUDE","DATE","TIME","ELEVATION","AZANGLE","ZENANGLE","SUNAZANGLE","SUNZENANGLE"]
        self.paramAtlas={}
        self.tmpFileOut=self.config.tmpFileOut
        self.tmpFileErr=self.config.tmpFileErr
        self.npcscores=200
        self.previousRunIsPc=False
        signal.signal(signal.SIGSEGV, self.sig_handler) 
        signal.signal(signal.SIGFPE, self.sig_handler) 
        signal.signal(signal.SIGTRAP , self.sig_handler) 
    def  sig_handler(self,signum, frame):
    #deal with the signal..
        msg="ERROR Something wrong occurred with RTTOV : see tmpFileErr.log log file and report the bug\n" 
        sys.stderr.write(msg)
        logging.ERROR(msg)
        
    def write (self,msg):    
        
        logging.info(msg) 

        print datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S"),msg    
                
    def beforeCallFortran(self):     
        # redirect stdin and stdout for fortran code
        # open 2 fds
     
        self.outFile= os.open(self.tmpFileOut, os.O_RDWR|os.O_CREAT|os.O_TRUNC)
        self.errFile = os.open(self.tmpFileErr, os.O_RDWR|os.O_CREAT|os.O_TRUNC)
        # save the current file stderr and stdout descriptors to a tuple
        self.save_fds = os.dup(1), os.dup(2)
        # put my files fds on 1 and 2O3_DATA      =>    False <type 'bool'>
        os.dup2(self.outFile, 1)
        os.dup2(self.errFile, 2)  
        
         
    def afterCallFortran(self):
        # restore standard out and err
        os.dup2(self.save_fds[0], 1)
        os.dup2(self.save_fds[1], 2)
        # close the temporary fds
        os.close(self.outFile)
        os.close(self.errFile)
        for fileName in (self.tmpFileOut,self.tmpFileErr) :
            f=open(fileName,"r")  
            lines=f.readlines()
            for line in lines :
                print line,
                logging.info(str(line))
            print

            f.close()
        os.close(self.save_fds[0])
        os.close(self.save_fds[1])   
                    
    def openAsciiProfile(self,filename):
        p = pProfile()

        erreur = p.loadProfileAscii(filename)

        if( erreur != 1 ):
            self.myProfile=p
            self.myProfile.controlProfile()
            self.myOption.default()
            self.loadProfile=True
        
            self.ctrlCoherence()
     
            pub.sendMessage("Profile CHANGED")
            pub.sendMessage("Options CHANGED")
            pub.sendMessage("Profile LOADED")
            logging.debug( "project send message Profile LOADED")
            
            return 0
        else :
            print (filename, "cannot read profile wrong format")     
            return 1

             
    def openProfile(self,filename,number=1):
        '''
        Open a Profile from a hdf5 filename
        '''
        self.write( "open profile "+str(filename)+" "+str(number))
        # re-initialize an empty profile 
        self.myProfile=pProfile()
        # read the profile
        try : 
            self.myProfile.loadProfileNumber(filename, number)
        except :
            self.write("cannot load profile")
            return 1    
        # control the values of the Profile regarding min and max values
        self.myProfile.controlProfile()
        # read the options
        try :
            f=h5py.File(filename,"r")  
            if 'OPTIONS' in f.keys():   
                h5 = f['/OPTIONS/']
                self.myOption.loadh5(h5) 
              #  h5.close()  
            else :
                self.myOption.default()
            f.close()
        except :
            return 1
        self.ctrlCoherence()
        self.loadProfile=True
        pub.sendMessage("Profile CHANGED")
        pub.sendMessage("Options CHANGED")
        pub.sendMessage("Profile LOADED")
        if self.useAtlas and self.myCoeffs.loadCoeffs :
            self.write("An atlas was used reinit Emissivity and Reflectance")
            self.reinitSurface()
            self.useAtlas=False
        logging.debug( "project send message Profile LOADED")
        return 0
        
    def saveProfile  (self,filename =""):
        """ save Profile and Option in filename """
        if (filename == ""): filename=self.config.profileDefaultFileName
        self.write( "\nSave profile to file: "+os.path.basename(filename))
        self.profileFileName=filename 
        myf = h5py.File(filename, 'w')
        self.myProfile.saveh5(myf,"/PROFILES/0001/") 

           
        self.myOption.saveh5(myf,"/OPTIONS/") 
        myf.close()
        del myf
          
        
    def updateProfile(self, aProfile):
        """ method to update self.myProfile sends a message "Profile CHANGED"
            it is preferable to use this method to update the profile """
        logging.debug( "project updateProfile"   ) 
        self.myProfile=aProfile
        self.ctrlCoherence()
        pub.sendMessage("Profile CHANGED")    
        pub.sendMessage("Options CHANGED")   
    
    def updateOptions(self, aOption):
        """ method to update self.myOption sends a message "Options CHANGED" 
            it is preferable to use this method to update the options """
        self.myOption=aOption
        pub.sendMessage("Options CHANGED")
    
    def updateEmissivity(self,aEmissivity):
        self.myEmissivity= aEmissivity  
        logging.debug( "sendMessage Surface CHANGED" )
        pub.sendMessage("Surface CHANGED")
        
    def updateReflectance(self,aReflectance):
        self.myReflectance= aReflectance
        logging.debug( "sendMessage Surface CHANGED" )
        pub.sendMessage("Surface CHANGED")
     
              
          
    def openSurface (self,filename,ctrl=True) :
        """ open an surface file and load it to the myEmissivity and myReflectance objects       
            must check if the dimension of EMIS and REFL of the file are coherent with number
            of channel loaded """
            
        f = h5py.File(filename, 'r')  
        h5 = f['/EMISSIVITY/']
        emissivity= rttov.emissivity.Emissivity()
        emissivity.loadh5(h5)
        if len(emissivity['EMIS_IN'].shape) == 0  :
            emisSize=1
        else :
            emisSize=emissivity['EMIS_IN'].shape[0]
            
        logging.debug( "nb channels = "+str(self.myCoeffs.nchannels)+ " try to load a file with : "+str(emisSize)+" channels")
        if ( ctrl ) :
            if emisSize != self.myCoeffs.nchannels :
                self.write( "incoherent number of channels : can't load surface file"+filename)
                return 1
                f.close()
        self.myEmissivity=emissivity

        if 'REFLECTANCE' in f.keys():
            logging.debug( "REFLECTANCE in surface")
            h5 = f['/REFLECTANCE/']
            self.myReflectance.loadh5(h5)  
        else :
            # create myReflectance with all zero 
            self.myReflectance= rttov.reflectance.Reflectance()
            self.myReflectance.setReflectance(emisSize)
        f.close()      
        logging.debug( "control Surface ")
        self.controleSurface()
        logging.debug( "openSurface send message Surface CHANGED")
        pub.sendMessage("Surface CHANGED")
           
    
        
    def saveSurface (self,filename) :
        """ save an surface file """
        if (filename == ""): filename=self.surfaceFileName 
        logging.debug("saveSurface filename"+str(filename))
        f =  h5py.File(filename, 'w')     
        self.myEmissivity.saveh5(f,'/EMISSIVITY/')
        self.myReflectance.saveh5(f, '/REFLECTANCE/')
        self.write( "file surface saved" +filename)   
        f.close()
        
    def reinitSurface(self):
        """ reinit the surface surface file with everything to 0 and cal to true """
        if not self.myCoeffs.loadCoeffs :
            sys.stderr.write("Cannot reinit surface coefficient file not loaded")
            self.write("Cannot reinit surface coefficient file not loaded")
            return        
        logging.debug("nchannels : "+ str( self.myCoeffs.nchannels))
        if self.myEmissivity["EMIS_IN"] != None :
            self.myEmissivity["EMIS_IN"][:]=0
            self.myEmissivity["EMIS_OUT"][:]=0
            self.myEmissivity["CALCEMIS"][:]=True
        if  self.myReflectance["REFL_IN"] != None :
            self.myReflectance["REFL_IN"][:]=0
            self.myReflectance["REFL_OUT"][:]=0
            self.myReflectance["CALCREFL"][:]=True
            self.saveSurface(self.surfaceFileName)
            logging.debug( "project.reinitSurface send message Surface Changed")
            pub.sendMessage("Surface CHANGED")
            
    def createSurface(self):
        """ create a surface file with everything to 0 and cal to true """
        if not self.myCoeffs.loadCoeffs :
            sys.stderr.write("Cannot create surface coefficient file not loaded")
            self.write("Cannot create surface coefficient file not loaded")
            return        
        logging.debug("nchannels : "+ str( self.myCoeffs.nchannels))
        self.myEmissivity=rttov.emissivity.Emissivity()
        self.myReflectance=rttov.reflectance.Reflectance()
        self.myEmissivity.setEmissivity(self.myCoeffs.nchannels)
        logging.debug( 'made new emissivity')
        self.myReflectance.setReflectance(self.myCoeffs.nchannels)
        logging.debug( "made new reflectance")
        self.saveSurface(self.surfaceFileName)  
        self.useAtlas=False   
        logging.debug( "project.createSurface send message Surface Changed")
        pub.sendMessage("Surface CHANGED")
        
    def controleSurface(self):
        """ controle the surface """
        msg="WARNING : there are suspect values of input emissivity or reflectance : reinit all values "
        if self.myCoeffs.loadCoeffs and self.loadProfile :
            if self.myEmissivity["EMIS_IN"] != None :
                if numpy.any(self.myEmissivity["EMIS_IN"] > 1 ) or  numpy.any(self.myEmissivity["EMIS_IN"] < 0 ):
                    self.write(msg)
                    self.myEmissivity["EMIS_IN"][:]=0
                    self.myEmissivity["CALCEMIS"][:]=True
                if numpy.all(self.myEmissivity["EMIS_IN"] == 0) and numpy.all(self.myEmissivity["CALCEMIS"] == False) :
                    self.write(msg)
                    self.myEmissivity["CALCEMIS"][:]=True
            if  self.myReflectance["REFL_IN"] != None and self.myCoeffs.hasSolar() :
                if numpy.any(self.myReflectance["REFL_IN"] > 1 ):
                    self.write(msg)
                    self.myReflectance["REFL_IN"][:]=0
                    self.myReflectance["CALCREFL"][:]=True
                if numpy.all(self.myReflectance["REFL_IN"] == 0) and  numpy.all(self.myReflectance["CALCREFL"] == False) :
                    self.write(msg)
                    self.myReflectance["CALCREFL"][:]=True
                  
                    
                  
    def newSurface(self):
        """ delete surface and re-initialise it"""
        self.myEmissivity["EMIS_IN"]=None
        self.myReflectance['REFL_IN']=None
        self.createSurface()
        
    def loadCoefficients(self,redirectOutput=True):
        """ on and load a coefficient file coefficient files must have been chosen"""
        err=-1
        self.myCoeffs.WAVENUMBERS=None
        previousNbChannels=self.myCoeffs.nchannels
        if redirectOutput : self.beforeCallFortran()
        for k in self.myCoeffs.fileName.keys() :
            if self.myCoeffs.fileName[k] != "" :
                if not os.path.isfile(self.myCoeffs.fileName[k]) :
                    self.write("file :"+self.myCoeffs.fileName[k]+ " does not exist ! ")
                    return 1
        self.myCoeffs.nchannels,err = rttov_gui_f2py.rttov_gui_load(self.myCoeffs.fileName["standard"],self.myCoeffs.fileName["aerosols"],self.myCoeffs.fileName["clouds"],self.myCoeffs.fileName["PC"])
        if redirectOutput : self.afterCallFortran()
        if err == 0 : 
            self.myCoeffs.loadCoeffs=True
            n = rttov.getcoefval.rttov_get_coef_val_r1('WAVENUMBERS')
            self.myCoeffs.WAVENUMBERS=n
            if len(n) <20 : self.write( "WAVENUMBERS "+str(n))
            self.write("rttov_gui_load ok nb channels="+str(self.myCoeffs.nchannels))
            if self.myCoeffs.nchannels != previousNbChannels :
                self.newSurface()       
                self.firstTimeSurface=True
            self.ctrlCoherence()
            pub.sendMessage("Options CHANGED")     
            n = rttov.getcoefval.rttov_get_coef_val_r1('REF_TEMPERATURE')
            if len(n) <20 :  self.write("REF_TEMPERATURE",+str(n))  
            pub.sendMessage("Coefficients CHANGED")         
        else : 
            self.myCoeffs.loadCoeffs=False
            sys.stderr.write("Cannot load coefficients \n")
            self.write("Cannot load coefficients")
            return
        
            
        return err       
    def dropCoefficients(self):
        """ drop a coefficients files coefficient files must have been loaded"""
        err=-1
        if self.myCoeffs.loadCoeffs :
            self.beforeCallFortran()
            err = rttov_gui_f2py.rttov_gui_drop()
            self.afterCallFortran()
            if err == 0 : 
                self.myCoeffs.loadCoeffs=False
                self.myCoeffs.nchannels=0
                self.write("drop coefficient file OK ")            
                self.ctrlCoherence()
                pub.sendMessage("Options CHANGED")   
                pub.sendMessage("Coefficients CHANGED") 
                self.myCoeffs.WAVENUMBERS=None
            else : 
                self.write("Cannot drop coefficient files ")   
                sys.stderr.write("Cannot drop coefficient files \n")
        else :
            self.write("Coefficient files not loaded ")
            sys.stderr.write("Coefficient files not loaded \n")
        return err       
    def runDirect(self):
        err=self.runModel(mode=self.RTTOV_MODE)
        return err
    
    def isMW(self):
        if self.loadProfile and self.myCoeffs.loadCoeffs :
            if (self.myCoeffs.isMW() ) :
                return True
            else :
                return False
        else :
            return False    
    def isSolar(self) :
        if self.loadProfile and self.myCoeffs.loadCoeffs :
            if (self.hasSolar() and self.myOption["ADDSOLAR"]) :
                return True
            else :
                return False
        else :
            return False    
    def hasSolar(self) :
        if self.loadProfile and self.myCoeffs.loadCoeffs :
            if (self.myCoeffs.hasSolar() ) :
                return True
            else :
                return False
        else :
            return False    
    
    def isPC(self):
        if self.loadProfile and self.myCoeffs.loadCoeffs :
            if (self.myCoeffs.hasPC() and self.myOption["ADDPC"]) :
                return True
            else :
                return False
        else :
            return False    
    def isClouds(self):
        if self.loadProfile and self.myCoeffs.loadCoeffs :
            if (self.hasClouds() and self.myOption["ADDCLOUDS"]) :
                return True
            else :
                return False
        else :
            return False 
    def hasClouds(self):
        if self.loadProfile and self.myCoeffs.loadCoeffs :
            if (self.myCoeffs.hasClouds() and self.myProfile.hasClouds()) :
                return True
            else :
                return False
        else :
            return False 
    
    def hasGas(self,gas):
        if self.loadProfile and self.myCoeffs.loadCoeffs :
            if self.myCoeffs.hasGas(gas) and self.myProfile.hasGas(gas) :
                return True
            else :
                return False
            
    def controleOptionGas(self):
        for gas in ( "O3","CO","CO2","CH4","N2O" ) :
            if self.hasGas(gas)    :
                self.myOption.statusOption[self.myOption.profileItemOptionName[gas]]=True
            else :
                self.myOption.statusOption[self.myOption.profileItemOptionName[gas]]=False
                self.myOption[self.myOption.profileItemOptionName[gas]]=False
                    
    def isAerosols(self):
        if self.loadProfile and self.myCoeffs.loadCoeffs :
            if (self.hasAerosols() and self.myOption["ADDAEROSL"])  :
                return True
            else :
                return False
        else :
            return False     
    def hasAerosols(self):
        if self.loadProfile and self.myCoeffs.loadCoeffs :
            if (self.myCoeffs.hasAerosols() and self.myProfile.hasAerosols()) :
                return True
            else :
                return False
        else :
            return False     
    
    def isPCClouds(self):
        if self.loadProfile and self.myCoeffs.loadCoeffs :
            if (self.myCoeffs.isPCClouds() and self.myOption["ADDPC"] and self.myOption["ADDCLOUDS"]) :
                return True
            else :
                return False
        else :
            return False
        
               
    def runModel(self,mode) :
        """ 
        save the profile and run the direct RTTOV model (a profile must exist )    
        """    
        err=-1
        if (not self.loadProfile ) : 
            sys.stderr.write("profile must be loaded \n") 
            err=-1
            return err 
        self.ctrlCoherence()
        pub.sendMessage("Options CHANGED")   
        self.saveProfile()
        logging.debug( "self.firstTimeSurface=  "+str(self.firstTimeSurface))
        if self.previousRunIsPc == True : 
            self.firstTimeSurface=True    
        print ">>self.firstTimeSurface",self.firstTimeSurface
        if self.firstTimeSurface :
            self.newSurface()
            self.fistTimeSurface=False
        else :
            self.saveSurface(self.surfaceFileName)
        print ">>self.firstTimeSurface",self.firstTimeSurface    
        logging.debug( "self.firstTimeSurface= "+str(self.firstTimeSurface))     
        self.write( "rttov_gui_f2py.rttov_gui_run with: ")
        self.write( str(self.profileFileName))
        self.write( str(self.surfaceFileName))
        self.write( str(self.radianceFileName))
        self.write( self.KMatrixFileName)
        self.write( self.transFileName)
        self.write ("mode :"+str(mode))
        self.write( "nthreads :"+str(self.nthreads))
        self.beforeCallFortran()
        #err = rttov_gui_f2py.rttov_gui_run(self.profileFileName,self.surfaceFileName,self.radianceFileName,self.KMatrixFileName,self.transFileName,mode,self.nthreads)
        try :
            err = rttov_gui_f2py.rttov_gui_run(self.profileFileName,self.surfaceFileName,self.radianceFileName,self.KMatrixFileName,self.transFileName,mode,self.nthreads)
            self.afterCallFortran()
            self.write( "rttov_gui_run completed return code : "+str(err))
        except :
            sys.stderr.write("ERROR with rttov_gui_f2py.rttov_gui_run\n")    
        if err == 0 : 
            self.write("rttov run mode "+mode+" ok \n")
            self.openSurface(self.surfaceFileName)
        else : 
            self.write("rttov run mode "+mode+" in error \n")
        self.previousRunIsPc=False
        return err
                     
    def runK(self):
        """  save the profile and run the direct RTTOV model (a profile must exist) """
        err=self.runModel(mode="K")
        return err
    
    def runPC(self):
        self.previousRunIsPc=True
        mode="PCDIRECT"
        err=self.runPCmodel(mode)
        return err
    
    def runPCK(self):
        self.previousRunIsPc=True
        mode="PCK"
        err=self.runPCmodel(mode)
        return err
   
    def runPCmodel(self,mode):
        """ run PC model """
        err=-1
        if (not self.loadProfile ) : 
            self.write("profile must be loaded \n") 
            err=-1
            return err 
        self.ctrlCoherence()
        pub.sendMessage("Options CHANGED")   
        if self.debug : self.myOption.display()
        self.saveProfile()
        logging.info("self.firstTimeSurface="+str(self.firstTimeSurface))
        logging.debug( "self.firstTimeSurface="+str(self.firstTimeSurface))
        if self.firstTimeSurface :
            self.createSurface()
            self.firstTimeSurface=False
        else :
            self.saveSurface(self.surfaceFileName)
        logging.debug( "self.firstTimeSurface="+str(self.firstTimeSurface))     
        self.write("rttov_gui_f2py.rttov_gui_pcrun with: ")
        self.write("  profile   : "+self.profileFileName)
        self.write("  surface   : "+self.surfaceFileName)
        self.write("  pc        : "+self.pcFileName)
        self.write("  pcKMatrix : "+self.pcKMatrixFileName)
        self.write("  mode      : "+mode)
        self.write("  threads   : "+str(self.nthreads))
        self.write("  npcscores :"+str(self.npcscores))
        self.beforeCallFortran()
        err=rttov_gui_f2py.rttov_gui_pcrun(self.profileFileName,self.surfaceFileName,self.pcFileName,self.pcKMatrixFileName,mode,self.nthreads,self.npcscores)
        self.afterCallFortran()
        self.write( "rttov_gui_pcrun completed return code : "+str(err))
        if err == 0 : 
            self.write("rttov run mode "+mode+" ok \n")
            self.openSurface(self.surfaceFileName,ctrl=False)
        else : 
            self.write("rttov run pc mode "+mode+" in error")
            sys.stderr.write("rttov run pc mode "+mode+" in error \n")
        return err
        
    
             
    def openAtlas(self):
        logging.debug( "project openAtlas")
        #pass
        if (not self.loadProfile ) : 
            sys.stderr.write("profile must be loaded \n") 
            err=-1
            return err 
        self.saveProfile()
        
        for param in self.ListeParamAtlas :
            self.paramAtlas[param]= self.myProfile[param]  
       
        self.write( "load Atlas for lat= "+str(self.paramAtlas['LATITUDE'])+" lon= "+str(self.paramAtlas['LONGITUDE']))
        self.createSurface()
        self.write( "rttov_gui_f2py.rttov_gui_get_emisbrdf with: ")
        self.write(self.config.ENV['RTTOV_GUI_EMISS_DIR']+","+self.profileFileName+","+self.surfaceFileName+",100,100,100")
        self.beforeCallFortran()
        err = rttov_gui_f2py.rttov_gui_get_emisbrdf(self.config.ENV['RTTOV_GUI_EMISS_DIR'],self.profileFileName,self.surfaceFileName,100,100,100)
        self.afterCallFortran()
        self.write( "load atlas completed : "+str(err))
        if err == 0 : 
            self.write("load atlas ok \n")
            self.openSurface(self.surfaceFileName) 
            self.useAtlas=True     
            self.firstTimeSurface=False       
        else : 
            self.write("cannot load atlas")
            sys.stderr.write("cannot load atlas \n")
        return err

     
    def displayRadiance(self):
        """ display the radiance file """
        if os.path.exists(self.radianceFileName) :
            f = h5py.File(self.radianceFileName, 'r')
            h5 = f['/RADIANCE/']
            r1 = rttov.radiance.Radiance()
            r1.loadh5(h5)
            f.close()
            r1.display()
            
    def ctrlCoherenceProfile(self):
        """ controle composant in the profile vs option """
        for item in self.myOption.profileItemOptionName.keys() :
            if self.myProfile[item] == None : 
                self.myOption.statusOption[self.myOption.profileItemOptionName[item]]=False
                self.myOption[self.myOption.profileItemOptionName[item]]=False
            else :
                self.myOption.statusOption[self.myOption.profileItemOptionName[item]]=True

    def ctrlCoherence(self) :
        """ control the coherence between options and profile and loaded coefficients"""
        """ options which can't be modified have a False status """
        """ warning : control order is very important !!!"""
        for opt in "IPCBND","IPCREG":
            if self.myOption[opt] < 1 : self.myOption[opt]=1
            
        for option in self.myOption.statusOption.keys() :
            self.myOption.statusOption[option]=False 
                # not done for the moment :   
          
        
        if self.myCoeffs.loadCoeffs == True :
            self.ctrlCoherenceProfile()
            for theme in ('General configuration options','General RT options','VIS/IR-only RT options',"Interpolation and vertical grid options" ) :
                for option in self.myOption.optionsThemesListe[theme] : 
                    self.myOption.statusOption[option]=True            
            self.controleOptionGas()
            if not self.myCoeffs.hasPC() : 
                self.myOption["ADDPC"]=False
                self.myOption.statusOption["ADDPC"]=False
            else :
                self.myOption.updatePCChoicesList(self.myCoeffs)
                self.myOption.statusOption["ADDPC"]=True

                if self.isPC() :
                    for param in ("IPCBND","IPCREG","ADDRADREC","CLDSTR_SIMPLE" ) :
                        self.myOption.statusOption[param]=True
                    for param in ("ADDAEROSL","ADDSOLAR","CO2_DATA","CO_DATA","N2O_DATA","CH4_DATA" ,"DO_NLTE_CORRECTION","USER_CLD_OPT_PARAM","CLDSTR_SIMPLE") :
                        #PC coef file is not compatible with cloudy simulations
                        self.myOption[param]=False
                        self.myOption.statusOption[param]=False
                    if self.myCoeffs.isPCClouds() and self.myProfile.hasClouds() :
                       self.myOption.statusOption["ADDCLOUDS"]=True 
                       
                    else :
                        self.myOption["ADDCLOUDS"]=False
                        self.myOption.statusOption["ADDCLOUDS"]=True         # ????                 
                        
                    
                    if self.myOption['ADDRADREC']==False :
                        self.myOption['SWITCHRAD']=False   
                        self.myOption.statusOption['SWITCHRAD']=False 
                    else :
                        self.myOption.statusOption['SWITCHRAD']=True     
                else :
                    self.myOption['ADDRADREC']=False
                    for param in ("IPCBND","IPCREG","ADDRADREC" ) :
                        self.myOption.statusOption[param]=False 
                    for param in ("ADDAEROSL","ADDSOLAR","O2_DATA","CO_DATA","N2O_DATA","CH4_DATA","DO_NLTE_CORRECTION" ) :   
                        self.myOption.statusOption[param]=True   
                        
            # Aerosols control
            if self.hasAerosols() :
                if  not self.myOption['ADDPC'] :  self.myOption.statusOption["ADDAEROSL"] = True
            else: 
                self.myOption.statusOption["ADDAEROSL"] = False
                self.myOption["ADDAEROSL"] = False  
                
            # cloud control       
            if self.myCoeffs.fileName["clouds"] != "" and  self.myProfile.anyCloud() :
                self.myOption.statusOption["ADDCLOUDS"] = True
                if self.myOption.statusOption["ADDCLOUDS"]== True :
                    self.myOption.statusOption["CLDSTR_THRESHOLD"]=True
                    if self.isPC() : 
                        self.myOption.statusOption["CLDSTR_SIMPLE"]=False
                    else :
                        self.myOption.statusOption["CLDSTR_SIMPLE"]=True                         
            else: 
                self.myOption.statusOption["ADDCLOUDS"] = False   
                self.myOption["ADDCLOUDS"]=False 
                self.myOption.statusOption["CLDSTR_SIMPLE"] = False   
                self.myOption["CLDSTR_SIMPLE"]=False                   
                self.myOption.statusOption["CLDSTR_THRESHOLD"] = False   

                
            # Micro Wavescontrol    
            if self.isMW() :
                for option in self.myOption.optionsThemesListe["MW-only RT options"] :
                    self.myOption.statusOption[option]=True
                for option in self.myOption.optionsThemesListe["VIS/IR-only RT options"] :
                        self.myOption.statusOption[option]=False
                for param in ("ADDAEROSL","ADDSOLAR","CO2_DATA","CO_DATA","N2O_DATA","CH4_DATA" ,"DO_NLTE_CORRECTION","USER_CLD_OPT_PARAM","ADDCLOUDS","CLDSTR_SIMPLE","USER_AER_OPT_PARAM") :
                        self.myOption[param]=False        
            else :
                for option in self.myOption.optionsThemesListe["MW-only RT options"] :
                    self.myOption.statusOption[option]=False   
                for option in   ( "CLW_DATA","DO_LAMBERTIAN"):
                    self.myOption[option]=False 
                    # sauf CLW
             
            if not self.hasSolar() :
                self.myOption.statusOption['ADDSOLAR']=False
                self.myOption['ADDSOLAR']=False
            
            if self.myProfile['CLW'] == None : 
                self.myOption.statusOption["CLW_DATA"]=False
            
            if self.myOption.statusOption["ADDCLOUDS"]== False or self.myOption["ADDCLOUDS"] == False :
                self.myOption.statusOption["CLDSTR_THRESHOLD"]=False
                self.myOption.statusOption["CLDSTR_SIMPLE"]=False


            for gas in ("N2O","CO","CO2","CH4","O3") :    
                if not self.hasGas(gas) :
                    self.myOption[self.myOption.profileItemOptionName[gas]]=False
                    self.myOption.statusOption[self.myOption.profileItemOptionName[gas]]=False
           
            # ADDSOLAR not compatible with PC 
            if self.myOption["ADDSOLAR"] :
                for param in ("ADDPC","ADDRADREC" ):
                    self.myOption[param]=False
                    self.myOption.statusOption[param]=False
                for param in ("ADDPC","IPCBND" ):
                    self.myOption.statusOption[param]=False
                               
            # not done for the moment
            for option in ("USER_AER_OPT_PARAM","USER_CLD_OPT_PARAM"):   
                self.myOption.statusOption[option]=False  
                self.myOption[option]=False
            # NLTE
            
            if  self.myOption.statusOption["DO_NLTE_CORRECTION"]==True :
                if not self.myCoeffs.hasNLTE() :
                    self.myOption.statusOption["DO_NLTE_CORRECTION"]=False
                    self.myOption["DO_NLTE_CORRECTION"]=False
                    
def OpenAProfile (filename,number):

    # read the profile
    profile=pProfile()
    if number == 1 :
        f=h5py.File(filename,"r")  
        h5 = f['/PROFILES/0001/']  
        profile.loadh5(h5)
        f.close()
    else :    
        profile.loadProfileNumber(filename, number)
    profile.controlProfile()   
    return profile            
            
if __name__ == "__main__":
    p=Project()
    print "Configuration : ",p.config.ENV['RTTOV_GUI_PREFIX']
    print "load coefficient"
    coefFile=p.config.ENV['RTTOV_GUI_COEFF_DIR']+"/rttov7pred54L/rtcoef_noaa_19_hirs.dat"
    cloudCoefFile=p.config.ENV['RTTOV_GUI_COEFF_DIR']+"/cldaer/"
    cloudCoefFile=p.config.ENV['RTTOV_GUI_COEFF_DIR']+"/cldaer/sccldcoef_noaa_19_hirs.dat"
    print coefFile
    print cloudCoefFile
    print "isPC ?",p.isPC()
    p.myCoeffs.fileName["standard"]=coefFile
    p.myCoeffs.fileName["clouds"]=cloudCoefFile                     
    print "isPC ?",p.isPC()
    p.loadCoefficients()
    print "isPC ?",p.isPC()
    p.myCoeffs.isMW()
    p.myCoeffs.hasSolar()
    for gas in ("CO2","O3","N2O","CO","CH4")  :
          print gas ,"has" , p.myCoeffs.hasGas(gas)
    print "isPC ?",p.isPC()
    print "change profile"
    #profileName=p.config.ENV["RTTOV_GUI_PREFIX"]+"/PROFILES/cld101lev_allgas.H5"
    profileName=p.config.ENV["RTTOV_GUI_PROFILE_DIR"]+"/AA_preproCf_44L.H5"
    print profileName
    p.openProfile(profileName) 
    p.myProfile.display()
    print "has cloud ?"
    print p.myProfile.anyCloud()
    print p.myProfile.cloud_list
    print p.myProfile.anyAerosol()
    
    
    print "test ctrlCoherenceClouds()"
    profileName=p.config.ENV["RTTOV_GUI_PROFILE_DIR"]+"/cld101lev_allgas.H5"
    print "has cloud ?"
    print p.myProfile.anyCloud()
    print p.myProfile.cloud_list
    print profileName
    p.openProfile(profileName)  
    print "has cloud ?"
    print p.myProfile.anyCloud()
    print p.myProfile.cloud_list
    p.myProfile.ctrlCoherenceClouds()
    p.myProfile.removeCloud("STCO")
    
    p.myProfile.ctrlCoherenceClouds()
    for cloud in  p.myProfile.cloud_list :
        p.myProfile.removeCloud(cloud)
    
    p.myProfile.ctrlCoherenceClouds() 
    for cloud in  p.myProfile.cloud_list :   
        p.myProfile.addCloud('STCO')
    print p.myProfile ['STCO']
    print p.myProfile ['CFRAC']
    p.myProfile.ctrlCoherenceClouds() 
    print p.myProfile ['STCO']
    print p.myProfile ['CFRAC']
    
             
    p.runDirect()
   
 
    p.displayRadiance()
    p.dropCoefficients()
    # test avec emissivity
    p.loadCoefficients()
    # test open Atlas
    p.debug=True
    p.openAtlas()
    coefFile=p.config.ENV['RTTOV_GUI_COEFF_DIR']+"/rttov9pred54L/rtcoef_metop_1_avhrr.dat"
   
    print coefFile
    print cloudCoefFile
    p.myCoeffs.fileName["standard"]=coefFile
    p.myCoeffs.fileName["clouds"]=""                    
    
    p.loadCoefficients()

    toto=p.myCoeffs.getFMV_GAS_POS()
    print "FMV_GAS_POS:",toto
    p.myCoeffs.isMW()
    p.myCoeffs.hasSolar()
    for gas in ("CO2","O3","N2O","CO","CH4")  :
        print gas ,"has" , p.myCoeffs.hasGas(gas)
    coefFile=p.config.ENV['RTTOV_GUI_COEFF_DIR']+"/rttov7pred54L/rtcoef_metop_1_hirs.dat"       
    p.myCoeffs.fileName["standard"]=coefFile
    p.loadCoefficients()
    print coefFile
    toto=p.myCoeffs.getFMV_GAS_POS()
    print "FMV_GAS_POS:",toto 
    coefFile=p.config.ENV['RTTOV_GUI_COEFF_DIR']+"/rttov7pred54L/rtcoef_metop_1_avhrr.dat"       
    p.myCoeffs.fileName["standard"]=coefFile
    p.loadCoefficients()
    print coefFile
    toto=p.myCoeffs.getFMV_GAS_POS()
    print "FMV_GAS_POS:",toto 
    p.dropCoefficients()
    print "------------------------------------------------------------------------------"
    coefFile=p.config.ENV['RTTOV_GUI_COEFF_DIR']+"/rttov8pred54L/rtcoef_metop_1_hirs.dat"
    p.myCoeffs.fileName["standard"]=coefFile
    p.loadCoefficients()
    print coefFile
    print "has solar ?"
    print p.myCoeffs.hasSolar()
    p.dropCoefficients()
    print "---------------------TEST PC -------------------------------------------------"
    coefFile=p.config.ENV['RTTOV_GUI_COEFF_DIR']+"/rttov9pred101L/rtcoef_metop_2_iasi.H5"
    pcFile=p.config.ENV['RTTOV_GUI_COEFF_DIR']+"/pc/pccoef_metop_2_iasi.H5"
    p.myCoeffs.fileName["standard"]=coefFile
    p.myCoeffs.fileName["PC"]=pcFile   
    print p.myCoeffs.fileName
    
    p.loadCoefficients()      
  
    print "isPC ?",p.isPC()   
  
    print "run PC model"
    
    p.myOption["ADDPC"]=True
    print "isPC ?",p.isPC()   
    print p.myCoeffs.fileName

    print "-------------------runPC------------------------"
    p.runPC()
    p.myOption['ADDPC']=False
    print "-------------------run direct ------------------" 
    p.myOption['ADDPC']=False
    p.saveProfile()
    print p.profileFileName,"",p.radianceFileName,p.KMatrixFileName,p.transFileName,"DIRECT",p.nthreads
     #rttov_gui_f2py.rttov_gui_run(p.profileFileName,"",p.radianceFileName,p.KMatrixFileName,p.transFileName,"DIRECT",p.nthreads)
    p.runDirect()
    print "--------------------runPCK------------------"
    p.myOption['ADDPC']=True
    p.runPCK()
