#!/usr/bin/perl -w

package RH;

use strict;

sub new
{
  my $class = shift;
  my $self = bless { @_ }, $class;
  return $self;
}

package WS;

use strict;
our @ISA = qw (RH);

sub compile
{
  my $self = shift;
  return "nohup @_ 1>/dev/null 2>/dev/null &";
}

sub run
{
  my $self = shift;
  return "nohup @_ 1>/dev/null 2>/dev/null &";
}

package SP;

use strict;
use File::Basename;
our @ISA = qw (RH);

sub compile
{
  my ($self, @args) = @_;
  my @opts = (
    -q => 'compile',
    -b => 1,
    -l => 'cputim_job=03:00:00,memsz_job=1500mb,elapstim_req=03:00:00',
    -j => 'o',
  );
  my ($base, $path) = fileparse ($args[-1]);
  $args[-1] = $base;
  return "cd $path && $self->{'target-qsub-path'} @opts @args";
}

sub run
{
  my ($self, @args) = @_;
  my @opts = (
    -q => 'vector',
    -l => 'cputim_job=04:00:00,memsz_job=120Gb,elapstim_req=04:30:00',
    -l => 'cpunum_job=8',
    -l => 'cpunum_prc=1',
    -j => 'o',
  );
#  my @opts = (
#    -q => 'vector',
#    -l => 'cputim_job=00:45:00,memsz_job=12Gb,elapstim_req=01:00:00',
#    -j => 'o',
#  );
  my ($base, $path) = fileparse ($args[-1]);
  $args[-1] = $base;
  return "cd $path && $self->{'target-qsub-path'} @opts @args";
}



package main;

use strict;
use File::Basename;
use File::Spec;
use File::Path;
use Data::Dumper;
use FileHandle;
use File::Temp;
use Getopt::Long;
use FindBin qw($Bin);

my @opts_s = (
  [ 'rttov-basedir',  'RTTOV main directory (local host)',                                    0, dirname ($Bin) ],
  [ 'arch',           'RTTOV arch (remote host)',                                             1, undef          ],
  [ 'test-list',      'RTTOV test list',                                                      0, ''             ],
  [ 'test-extra-opts','RTTOV test options (will be passed on the command line)',              0, ''             ],
  [ 'email',          'Email to send log file to',                                            0, ''             ],
  [ 'comment',        'Comment',                                                              0, ''             ],
  [ 'make-target',    'makefile target',                                                      0, 'all'          ],
  [ 'makefilepl-opts',    'Makefile.PL options',                                              0, ''             ],


  [ 'target',         'user@host (remote machine)',                                           1, undef          ],
  [ 'target-dir',     'target directory (remote host, where RTTOV is to be installed)',       1, undef          ],
  [ 'target-coefdir', 'coefficient directory (remote host)',                                  1, undef          ],
  [ 'target-class',   'SP (supercomputer), WS (workstation)',                                 0, 'WS'           ],
  [ 'target-rsync-path', 'Path of rsync (remote host)',                                       0, undef          ],
  [ 'target-shell-path', 'Path of bash or ksh (remote host)',                                 0, '/bin/bash'    ],
  [ 'target-qsub-path',  'Path of qsub (remote host)',                                        0, 'qsub'         ],
);


my @opts_f = (
  [ 'make-clean',     "execute `make clean' before building RTTOV",  ''  ],
  [ 'help',           'Display help message',                        ''  ],
);

sub help
{
  die ("Usage:$0\n" .
    join ('', map { sprintf ("  --%-30s %s\n", $_->[0], $_->[1]) } @opts_s),
    join ('', map { sprintf ("  --%-30s %s\n", $_->[0], $_->[1]) } @opts_f),
  );
}


@ARGV or &help ();

my %opts; 

GetOptions (
  map ({ ("$_->[0]=s", \$opts{$_->[0]}) } @opts_s),
  map ({ ("$_->[0]",   \$opts{$_->[0]}) } @opts_f),
) or &help ();

$opts{help} && &help ();

$ENV{RTTOV_REMOTE_CONF_DIR} ||= '.';

# .conf defaults
my %opts1 = %{my $conf = "$ENV{RTTOV_REMOTE_CONF_DIR}/$opts{target}.conf"; (-f $conf ? do ("$conf") : { })};
while (my ($key, $val) = each (%opts1))
  {
    $opts{$key} ||= $val;
  }

# defaults
my %opts2 = ((map ({ ($_->[0], $_->[3]) } @opts_s)), (map ({ ($_->[0], $_->[2]) } @opts_f)));
while (my ($key, $val) = each (%opts2))
  {
    $opts{$key} ||= $val;
  }


for my $opt (@opts_s)
  {
    next unless ($opt->[2]);
    next if ($opts{$opt->[0]});
    die ("Option --$opt->[0] ($opt->[1]) is mandatory\n");
  }


my $host = $opts{'target-class'}->new (%opts);


my ($base, $dir) = fileparse ($opts{'rttov-basedir'});

chdir ($dir);

(-d 'tmp') or mkpath ('tmp');

my $tmp_com = 'File::Temp'->new ( UNLINK => 0, SUFFIX => '.sh', DIR => 'tmp' );
my $tmp_run = 'File::Temp'->new ( UNLINK => 0, SUFFIX => '.sh', DIR => 'tmp' );
my $sh_com = $tmp_com->filename ();
my $sh_run = $tmp_run->filename ();
chmod (0755, $_) for ($sh_com, $sh_run);


# liste des repertoires et fichiers a synchroniser
#  test_emisatlas.1 et test_rttovscatt.1
#  devraient etre dans tests.0
# on peut aussi trouver une autre solution pour emis_data
my @subdirs = map ({ 'File::Spec'->canonpath ($_) }
                map ({ "$base/$_" } qw (VERSION src build rttov_test/rttov_test.pl rttov_test/tests.0 rttov_test/profile-datasets)),
                "$base/rttov_test/arch", <$base/rttov_test/*.2>, <$base/rttov_test/*.sh>, $sh_com, $sh_run,
		<$base/rttov_test/test_example_fwd.1>, <$base/rttov_test/test_example_pc_fwd.1>, <$base/rttov_test/test_emisatlas.1>, <$base/rttov_test/test_rttovscatt.1>  );

my @test_list = map { "$_.sh" } split (m/,/o, $opts{'test-list'});


$_->print (<< "EOF") for ($tmp_com, $tmp_run);
#!$opts{'target-shell-path'}

set -x


export EMAIL=$opts{email}
export COMMENT="$opts{comment}"
log=$opts{'target-dir'}/$_.eo

(

set -x


export ARCH=$opts{arch}
export RTTOV_BASEDIR=$opts{'target-dir'}/$base
export RTTOV_COEFDIR=$opts{'target-coefdir'}
export RTTOV_TEST_LIST="@test_list"
export RTTOV_TEST_OPTS="$opts{'test-extra-opts'}"
export RTTOV_MAKE_CLEAN=$opts{'make-clean'}
export RTTOV_MAKE_TARGET=$opts{'make-target'}
export RTTOV_MAKEFILEPL_OPTS="$opts{'makefilepl-opts'}"

if [ -f "\$HOME/.rttovrc" ]
then
  . "\$HOME/.rttovrc"
fi

if [ -f "$opts{'target-dir'}/.rttovrc" ]
then
  . "$opts{'target-dir'}/.rttovrc"
fi

EOF


$tmp_com->print (<< 'EOF');

# build

cd $RTTOV_BASEDIR/src || exit 1
../build/Makefile.PL $RTTOV_MAKEFILEPL_OPTS || exit 1

if [ "x$RTTOV_MAKE_CLEAN" != "x" ]
then
  make ARCH=$ARCH INSTALLDIR=install/$ARCH clean
fi

if [ -f "$MAKEFILE_LOCAL" ]
then
  OPT_LOCAL="MAKEFILE_LOCAL=$MAKEFILE_LOCAL"
else
  unset OPT_LOCAL
fi

make ARCH=$ARCH $OPT_LOCAL INSTALLDIR=install/$ARCH $RTTOV_MAKE_TARGET || exit 1
EOF

$tmp_com->print ($host->run ("$opts{'target-dir'}/$sh_run"));

$tmp_run->print (<< 'EOF');

# coef

cd $RTTOV_BASEDIR || exit 1

for v in 7 8 9 10 11
do
  \rm -f rtcoef_rttov$v
  if [ -d $RTTOV_COEFDIR/rttov$v ] 
  then 
     \ln -s $RTTOV_COEFDIR/rttov$v rtcoef_rttov$v
  fi

  if [ -d $RTTOV_COEFDIR/rtcoef_rttov$v ] 
  then
     \rm -f rtcoef_rttov$v
     \ln -s $RTTOV_COEFDIR/rtcoef_rttov$v rtcoef_rttov$v
  fi
done


# test

cd $RTTOV_BASEDIR/rttov_test || exit 1

if [ "x$RTTOV_TEST_LIST" == "x" ]
then
  RTTOV_TEST_LIST="$(ls *.sh)"
fi

EOF

$tmp_run->print (<< "EOF");

for sh in \$RTTOV_TEST_LIST
do
  $opts{'target-shell-path'} ./\$sh ARCH=\$ARCH BIN=install/\$ARCH/bin "\$RTTOV_TEST_OPTS"
done


EOF

$_->print (<< 'EOF') for ($tmp_com, $tmp_run);

) > $log 2>&1

cat $log

if [ "x$EMAIL" != "x" ]
then
  ( echo $COMMENT ; cat $log ) | mail $EMAIL
fi

EOF



my @rsync = (qw (rsync -r -u --delete -azR), 
             ($opts{'target-rsync-path'} ?  ("--rsync-path=$opts{'target-rsync-path'}") : ()), 
             @subdirs, "$opts{target}:$opts{'target-dir'}");
system (@rsync)
  and die ("Cannot @rsync");



my @ssh = (qw (ssh), $opts{target}, $host->compile ("$opts{'target-dir'}/$sh_com"));
system (@ssh)
  and die ("Cannot @ssh");








