#!/bin/sh
#
# IR emissivity atlas test control script
#
# 14/06/2010   Created (J. Hocking)
#

# Set BIN directory if supplied
BIN=`perl -e 'for(@ARGV){m/^BIN=(\S+)$/o && print "$1";}' $*`
if [ "x$BIN" = "x" ]
then
  BIN=bin
fi

######## Edit this section for your pathnames #######

# Relative to the rttov_test/test_emisatlas.1 directory:
DATART=../../rtcoef_rttov11/rttov7pred54L
DATAATLAS=../../emis_data
EXEC=../../$BIN/rttov_iratlas_test.exe
REF_TEST=../test_emisatlas.2

# Relative to the rttov_test directory:
TEST=./test_emisatlas.1

TEST_INPUT_FILE="rttov_iratlas_test_input"
MONTH=08
# Coefficient filenames are specified below

###################################################

ARG_ARCH=`perl -e 'for(@ARGV){m/^ARCH=(\S+)$/o && print "$1";}' $*`
if [ ! "x$ARG_ARCH" = "x" ]; then
  ARCH=$ARG_ARCH
fi
if [ "x$ARCH" = "x" ];
then
  echo 'Please supply ARCH'
  exit 1
fi

ATLAS_EMIS_FILE="UWirbfemis_COEF_V2.1_0.1deg_2007${MONTH}_mask.nc"
ATLAS_COV_FILE="UWiremis_hsremis_covmat_V1.0_deg0.5_month${MONTH}_mask.nc"
ATLAS_EIGVEC_FILE="UWiremis_labeigvects.nc"

if [ ! -d $TEST ]; then
  echo 'Test directory required, containing profiles_ir'
  echo 'Also, make sure to run directly from the rttov_test directory'
  exit
fi

cwd=`pwd`
cd $TEST

echo " "
echo " "
echo " Test IR emissivity atlas "
echo " "

for TEST_NUMBER in 01 02
# for TEST_NUMBER in 01
do

  case $TEST_NUMBER in
  
    '01') COEF_FILENAME=rtcoef_eos_1_modis.dat
   
    cat  > $TEST_INPUT_FILE  <<EOF
$MONTH                                    # Month for which to load emissivity data
$COEF_FILENAME
16                                        # Number of channels
1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16    # Channel list
$DATAATLAS
EOF

    ;;

    '02') COEF_FILENAME=rtcoef_metop_2_iasi.dat

    cat  > $TEST_INPUT_FILE  <<EOF
$MONTH
$COEF_FILENAME
19
93 137 185 237 285 349 413 1013 1533 2289 2877 3357 6181 6273 6381 6525 6865 7469 8105
$DATAATLAS
EOF

    ;;
  esac

  rm -f $COEF_FILENAME
  if [ -s  $DATART/$COEF_FILENAME ] ; then
    ln -s $DATART/$COEF_FILENAME $COEF_FILENAME
  else
    echo "rtcoef file not found"
    cd $cwd
    exit
  fi

  if [ ! -s $DATAATLAS/$ATLAS_EMIS_FILE ] ; then
    echo "Atlas emissivity file not found"
    cd $cwd
    exit
  fi
  
  if [ ! -s $DATAATLAS/$ATLAS_COV_FILE ] ; then
    echo "Atlas covariance file not found"
    cd $cwd
    exit
  fi

  if [ ! -s $DATAATLAS/$ATLAS_EIGVEC_FILE ] ; then
    echo "Atlas eigenvector file not found"
    cd $cwd
    exit
  fi
  
  $EXEC < $TEST_INPUT_FILE

  if [ $? -ne 0 ]; then
    echo " "
    echo "TEST FAILED"
    echo " "
    exit 1
  fi

  mv output_iratlas.ascii  output_iratlas.$TEST_NUMBER.$ARCH

  if [ $? -ne 0 ]; then
    echo "Expected output file not found"
    exit 1
  fi

  echo
  echo "Output is in the file ${TEST}/output_iratlas.$TEST_NUMBER.$ARCH"

  DIFF_FILE=diff_iratlas.$TEST_NUMBER.$ARCH

  if [ -f $REF_TEST/output_iratlas.$TEST_NUMBER ] ; then
    diff -biw output_iratlas.$TEST_NUMBER.$ARCH \
      $REF_TEST/output_iratlas.$TEST_NUMBER > $DIFF_FILE

    if [ `stat -c %s ${DIFF_FILE}` -eq 0 ]; then
      echo " "
      echo "Diff file has zero size: TEST SUCCESSFUL"
      echo " "
    else
      echo "--- Diff file contents: ---"
      cat $DIFF_FILE
      echo "---------------------------"
    fi
  else
    echo "Test reference output not found"
  fi
  echo

  rm -f $TEST_INPUT_FILE
  rm -f $COEF_FILENAME

done

cd $cwd

exit
