!
MODULE mod_rttov_baran2013_icldata

  ! Description:
  ! Constant coefficients and parameters for Baran Ice cloud parameters
  !
  ! Copyright:
  !    This software was developed within the context of
  !    the EUMETSAT Satellite Application Facility on
  !    Numerical Weather Prediction (NWP SAF), under the
  !    Cooperation Agreement dated 25 November 1998, between
  !    EUMETSAT and the Met Office, UK, by one or more partners
  !    within the NWP SAF. The partners in the NWP SAF are
  !    the Met Office, ECMWF, KNMI and MeteoFrance.
  !
  !    Copyright 2012, EUMETSAT, All Rights Reserved.
  !
  ! Current Code Owner: SAF NWP
  !
  ! History:
  ! Version   Date     Comment
  ! -------   ----     -------
  !  1.0       Oct 2012  New F90 code (P. Brunel J. Vidot)
  !  1.1       Mar 2013  New parameterization with 6 parameters 
  !                      for absorption and scattering coefficients (J.Vidot)
  !  1.2       Apr 2013  Add iwc and temperature limits from baran database (J. Vidot)   
  !
  ! Code Description:
  !   Language:           Fortran 90.
  !   Software Standards: "European Standards for Writing and
  !     Documenting Exchangeable Fortran 90 Code".
  !
  ! Declarations:
  ! Modules used:
  !
  USE parkind1, Only : jprb, jpim
  IMPLICIT NONE

  ! Environment setup
  ! -----------------
  ! Module use

  INTEGER(KIND=JPIM), PUBLIC, PARAMETER :: n_baran2013_wn      = 57_JPIM
  INTEGER(KIND=JPIM), PUBLIC, PARAMETER :: n_baran2013_abs     =  6_JPIM
  INTEGER(KIND=JPIM), PUBLIC, PARAMETER :: n_baran2013_sca     =  6_JPIM
  INTEGER(KIND=JPIM), PUBLIC, PARAMETER :: n_baran2013_bpr     =  3_JPIM
  INTEGER(KIND=JPIM), PUBLIC, PARAMETER :: n_baran2013_asym    =  3_JPIM
  INTEGER(KIND=JPIM), PUBLIC, PARAMETER :: n_baran2013_corbpr  =  6_JPIM
  INTEGER(KIND=JPIM), PUBLIC, PARAMETER :: n_baran2013_corasym =  6_JPIM

  REAL(KIND=JPRB), PUBLIC, PARAMETER :: baran2013_wvl(n_baran2013_wn) = (/ &
& 18.180_JPRB, 17.240_JPRB, 16.670_JPRB, 16.130_JPRB, 15.630_JPRB,  &
& 15.150_JPRB, 14.710_JPRB, 14.290_JPRB, 14.080_JPRB, 13.700_JPRB,  &
& 13.510_JPRB, 13.330_JPRB, 12.990_JPRB, 12.820_JPRB, 12.500_JPRB,  &
& 12.200_JPRB, 11.900_JPRB, 11.630_JPRB, 11.360_JPRB, 11.000_JPRB,  &
& 10.750_JPRB, 10.530_JPRB, 10.310_JPRB, 10.200_JPRB, 10.000_JPRB,  &
&  9.804_JPRB,  9.259_JPRB,  9.091_JPRB,  8.696_JPRB,  8.475_JPRB,  &
&  8.333_JPRB,  8.065_JPRB,  7.812_JPRB,  7.463_JPRB,  7.246_JPRB,  &
&  7.042_JPRB,  6.757_JPRB,  6.494_JPRB,  6.410_JPRB,  6.329_JPRB,  &
&  6.250_JPRB,  6.061_JPRB,  5.814_JPRB,  5.747_JPRB,  5.556_JPRB,  &
&  5.263_JPRB,  5.200_JPRB,  5.000_JPRB,  4.900_JPRB,  4.500_JPRB,  &
&  4.300_JPRB,  3.925_JPRB,  3.450_JPRB,  3.275_JPRB,  3.188_JPRB,  &
&  3.138_JPRB,  3.037_JPRB/)

  REAL(KIND=JPRB), PUBLIC, PARAMETER :: baran2013_wvn(n_baran2013_wn) =(/ &
&  550.05_JPRB,  580.05_JPRB,  599.88_JPRB,  619.96_JPRB,  639.80_JPRB,  &
&  660.07_JPRB,  679.81_JPRB,  699.79_JPRB,  710.23_JPRB,  729.93_JPRB,  &
&  740.19_JPRB,  750.19_JPRB,  769.82_JPRB,  780.03_JPRB,  800.00_JPRB,  &
&  819.67_JPRB,  840.34_JPRB,  859.85_JPRB,  880.28_JPRB,  909.09_JPRB,  &
&  930.23_JPRB,  949.67_JPRB,  969.93_JPRB,  980.39_JPRB, 1000.00_JPRB,  &
& 1019.99_JPRB, 1080.03_JPRB, 1099.99_JPRB, 1149.95_JPRB, 1179.94_JPRB,  &
& 1200.05_JPRB, 1239.93_JPRB, 1280.08_JPRB, 1339.94_JPRB, 1380.07_JPRB,  &
& 1420.05_JPRB, 1479.95_JPRB, 1539.88_JPRB, 1560.06_JPRB, 1580.03_JPRB,  &
& 1600.00_JPRB, 1649.89_JPRB, 1719.99_JPRB, 1740.04_JPRB, 1799.86_JPRB,  &
& 1900.06_JPRB, 1923.08_JPRB, 2000.00_JPRB, 2040.82_JPRB, 2222.22_JPRB,  &
& 2325.58_JPRB, 2547.77_JPRB, 2898.55_JPRB, 3053.44_JPRB, 3136.76_JPRB,  &
& 3186.74_JPRB, 3292.72_JPRB/)

  REAL(KIND=JPRB), PUBLIC, PARAMETER :: baran2013_regcoef_abs(n_baran2013_abs, n_baran2013_wn) = &
  RESHAPE( (/ &
&        1.5314788_JPRB,  -0.0017474_JPRB,   0.8827825_JPRB,   0.0000033_JPRB,   0.0104642_JPRB,   0.0004999_JPRB,  &
&        2.0604036_JPRB,  -0.0064955_JPRB,   0.7999442_JPRB,   0.0000129_JPRB,   0.0106611_JPRB,   0.0007330_JPRB,  &
&        1.6622337_JPRB,  -0.0018134_JPRB,   0.8767689_JPRB,   0.0000019_JPRB,   0.0097163_JPRB,   0.0004327_JPRB,  &
&        1.7057686_JPRB,  -0.0021751_JPRB,   0.8672317_JPRB,   0.0000026_JPRB,   0.0100666_JPRB,   0.0004654_JPRB,  &
&        1.7934082_JPRB,  -0.0028799_JPRB,   0.8517848_JPRB,   0.0000039_JPRB,   0.0104982_JPRB,   0.0005117_JPRB,  &
&        1.8764248_JPRB,  -0.0035748_JPRB,   0.8336809_JPRB,   0.0000054_JPRB,   0.0112608_JPRB,   0.0005762_JPRB,  &
&        1.9536288_JPRB,  -0.0042376_JPRB,   0.8155645_JPRB,   0.0000067_JPRB,   0.0120990_JPRB,   0.0006414_JPRB,  &
&        2.0086148_JPRB,  -0.0047284_JPRB,   0.8001392_JPRB,   0.0000077_JPRB,   0.0131068_JPRB,   0.0007050_JPRB,  &
&        2.0232859_JPRB,  -0.0048686_JPRB,   0.7946775_JPRB,   0.0000081_JPRB,   0.0135370_JPRB,   0.0007299_JPRB,  &
&        2.0359902_JPRB,  -0.0050052_JPRB,   0.7882498_JPRB,   0.0000084_JPRB,   0.0142356_JPRB,   0.0007636_JPRB,  &
&        2.0351632_JPRB,  -0.0050139_JPRB,   0.7859606_JPRB,   0.0000085_JPRB,   0.0145325_JPRB,   0.0007774_JPRB,  &
&        2.0347366_JPRB,  -0.0050240_JPRB,   0.7845127_JPRB,   0.0000085_JPRB,   0.0148049_JPRB,   0.0007877_JPRB,  &
&        2.0194855_JPRB,  -0.0049420_JPRB,   0.7823519_JPRB,   0.0000085_JPRB,   0.0153860_JPRB,   0.0008100_JPRB,  &
&        2.0103099_JPRB,  -0.0048776_JPRB,   0.7821797_JPRB,   0.0000084_JPRB,   0.0155639_JPRB,   0.0008157_JPRB,  &
&        1.9937268_JPRB,  -0.0047396_JPRB,   0.7839088_JPRB,   0.0000081_JPRB,   0.0157096_JPRB,   0.0008147_JPRB,  &
&        1.9646682_JPRB,  -0.0045013_JPRB,   0.7881347_JPRB,   0.0000077_JPRB,   0.0158035_JPRB,   0.0008070_JPRB,  &
&        1.9249202_JPRB,  -0.0041685_JPRB,   0.7940730_JPRB,   0.0000071_JPRB,   0.0157654_JPRB,   0.0007928_JPRB,  &
&        1.8797990_JPRB,  -0.0037748_JPRB,   0.8032467_JPRB,   0.0000063_JPRB,   0.0155613_JPRB,   0.0007649_JPRB,  &
&        1.8217283_JPRB,  -0.0032681_JPRB,   0.8150781_JPRB,   0.0000053_JPRB,   0.0152488_JPRB,   0.0007277_JPRB,  &
&        1.6808209_JPRB,  -0.0020152_JPRB,   0.8450431_JPRB,   0.0000028_JPRB,   0.0140292_JPRB,   0.0006238_JPRB,  &
&        1.5238456_JPRB,  -0.0006164_JPRB,   0.8813558_JPRB,  -0.0000001_JPRB,   0.0125364_JPRB,   0.0004940_JPRB,  &
&        1.3288318_JPRB,   0.0010385_JPRB,   0.9220192_JPRB,  -0.0000035_JPRB,   0.0110928_JPRB,   0.0003558_JPRB,  &
&        1.1538949_JPRB,   0.0025083_JPRB,   0.9579487_JPRB,  -0.0000065_JPRB,   0.0097124_JPRB,   0.0002303_JPRB,  &
&        1.0808266_JPRB,   0.0031167_JPRB,   0.9729619_JPRB,  -0.0000077_JPRB,   0.0091816_JPRB,   0.0001782_JPRB,  &
&        0.9348922_JPRB,   0.0043030_JPRB,   0.9995300_JPRB,  -0.0000101_JPRB,   0.0083708_JPRB,   0.0000919_JPRB,  &
&        0.8960238_JPRB,   0.0046158_JPRB,   1.0079271_JPRB,  -0.0000108_JPRB,   0.0080569_JPRB,   0.0000617_JPRB,  &
&        0.8807639_JPRB,   0.0047072_JPRB,   1.0108042_JPRB,  -0.0000110_JPRB,   0.0078271_JPRB,   0.0000499_JPRB,  &
&        0.8920884_JPRB,   0.0045956_JPRB,   1.0077935_JPRB,  -0.0000107_JPRB,   0.0079657_JPRB,   0.0000620_JPRB,  &
&        0.9510488_JPRB,   0.0040977_JPRB,   0.9975035_JPRB,  -0.0000097_JPRB,   0.0081696_JPRB,   0.0000932_JPRB,  &
&        0.9907174_JPRB,   0.0037715_JPRB,   0.9910332_JPRB,  -0.0000091_JPRB,   0.0083175_JPRB,   0.0001124_JPRB,  &
&        1.0172625_JPRB,   0.0035564_JPRB,   0.9864221_JPRB,  -0.0000087_JPRB,   0.0082961_JPRB,   0.0001242_JPRB,  &
&        1.0724044_JPRB,   0.0031033_JPRB,   0.9765334_JPRB,  -0.0000078_JPRB,   0.0084255_JPRB,   0.0001529_JPRB,  &
&        1.1442192_JPRB,   0.0025169_JPRB,   0.9646400_JPRB,  -0.0000066_JPRB,   0.0086534_JPRB,   0.0001878_JPRB,  &
&        1.2820891_JPRB,   0.0013761_JPRB,   0.9387301_JPRB,  -0.0000043_JPRB,   0.0093225_JPRB,   0.0002703_JPRB,  &
&        1.3675654_JPRB,   0.0006961_JPRB,   0.9248517_JPRB,  -0.0000030_JPRB,   0.0095327_JPRB,   0.0003081_JPRB,  &
&        1.4240755_JPRB,   0.0002130_JPRB,   0.9115443_JPRB,  -0.0000020_JPRB,   0.0101727_JPRB,   0.0003583_JPRB,  &
&        1.4998862_JPRB,  -0.0004303_JPRB,   0.8958891_JPRB,  -0.0000007_JPRB,   0.0108551_JPRB,   0.0004149_JPRB,  &
&        1.5313557_JPRB,  -0.0007048_JPRB,   0.8872254_JPRB,  -0.0000001_JPRB,   0.0112393_JPRB,   0.0004481_JPRB,  &
&        1.5475339_JPRB,  -0.0008405_JPRB,   0.8841819_JPRB,   0.0000002_JPRB,   0.0114176_JPRB,   0.0004596_JPRB,  &
&        1.5615413_JPRB,  -0.0009608_JPRB,   0.8808945_JPRB,   0.0000005_JPRB,   0.0115859_JPRB,   0.0004722_JPRB,  &
&        1.5786166_JPRB,  -0.0011014_JPRB,   0.8774281_JPRB,   0.0000007_JPRB,   0.0117390_JPRB,   0.0004845_JPRB,  &
&        1.5792314_JPRB,  -0.0011058_JPRB,   0.8759080_JPRB,   0.0000008_JPRB,   0.0118634_JPRB,   0.0004923_JPRB,  &
&        1.3756465_JPRB,   0.0006088_JPRB,   0.9182664_JPRB,  -0.0000027_JPRB,   0.0103588_JPRB,   0.0003466_JPRB,  &
&        1.2645241_JPRB,   0.0015312_JPRB,   0.9398013_JPRB,  -0.0000046_JPRB,   0.0096568_JPRB,   0.0002749_JPRB,  &
&        0.9581595_JPRB,   0.0040352_JPRB,   0.9962777_JPRB,  -0.0000096_JPRB,   0.0082046_JPRB,   0.0000958_JPRB,  &
&        0.6162041_JPRB,   0.0066874_JPRB,   1.0505459_JPRB,  -0.0000148_JPRB,   0.0073791_JPRB,  -0.0000553_JPRB,  &
&        0.5705082_JPRB,   0.0070107_JPRB,   1.0564741_JPRB,  -0.0000153_JPRB,   0.0075321_JPRB,  -0.0000644_JPRB,  &
&        0.5890197_JPRB,   0.0068613_JPRB,   1.0542367_JPRB,  -0.0000150_JPRB,   0.0074206_JPRB,  -0.0000620_JPRB,  &
&       -0.3340103_JPRB,   0.0163459_JPRB,   1.4274904_JPRB,  -0.0000392_JPRB,  -0.0252097_JPRB,  -0.0019216_JPRB,  &
&        0.2149816_JPRB,   0.0121874_JPRB,   1.3230900_JPRB,  -0.0000312_JPRB,  -0.0217038_JPRB,  -0.0015919_JPRB,  &
&        0.0307329_JPRB,   0.0136413_JPRB,   1.3602626_JPRB,  -0.0000341_JPRB,  -0.0232089_JPRB,  -0.0017174_JPRB,  &
&       -0.3653250_JPRB,   0.0165384_JPRB,   1.4325900_JPRB,  -0.0000396_JPRB,  -0.0254899_JPRB,  -0.0019384_JPRB,  &
&        0.3098240_JPRB,   0.0114014_JPRB,   1.3040057_JPRB,  -0.0000297_JPRB,  -0.0211393_JPRB,  -0.0015330_JPRB,  &
&        0.6174046_JPRB,   0.0089202_JPRB,   1.2206776_JPRB,  -0.0000251_JPRB,  -0.0161968_JPRB,  -0.0012292_JPRB,  &
&        0.6373657_JPRB,   0.0085417_JPRB,   1.2059457_JPRB,  -0.0000239_JPRB,  -0.0142528_JPRB,  -0.0011340_JPRB,  &
&        0.6334623_JPRB,   0.0084298_JPRB,   1.1993412_JPRB,  -0.0000235_JPRB,  -0.0135162_JPRB,  -0.0010935_JPRB,  &
&        0.5795429_JPRB,   0.0084684_JPRB,   1.1989553_JPRB,  -0.0000234_JPRB,  -0.0131293_JPRB,  -0.0010804_JPRB/),&
&     SHAPE =(/ n_baran2013_abs, n_baran2013_wn/))

  REAL(KIND=JPRB), PUBLIC, PARAMETER :: baran2013_regcoef_sca(n_baran2013_sca, n_baran2013_wn) =  &
  RESHAPE( (/ &
&        2.6999326_JPRB,  -0.0099672_JPRB,   0.7423267_JPRB,   0.0000177_JPRB,   0.0074778_JPRB,   0.0007242_JPRB,  &
&        2.8101315_JPRB,  -0.0116112_JPRB,   0.7110062_JPRB,   0.0000219_JPRB,   0.0092925_JPRB,   0.0008870_JPRB,  &
&        2.1961653_JPRB,  -0.0051626_JPRB,   0.8188832_JPRB,   0.0000077_JPRB,   0.0083949_JPRB,   0.0005210_JPRB,  &
&        2.1662555_JPRB,  -0.0049909_JPRB,   0.8165070_JPRB,   0.0000075_JPRB,   0.0093628_JPRB,   0.0005547_JPRB,  &
&        2.1030188_JPRB,  -0.0046849_JPRB,   0.8121740_JPRB,   0.0000074_JPRB,   0.0112321_JPRB,   0.0006261_JPRB,  &
&        1.9877878_JPRB,  -0.0038026_JPRB,   0.8152618_JPRB,   0.0000058_JPRB,   0.0122263_JPRB,   0.0006499_JPRB,  &
&        1.9665515_JPRB,  -0.0037087_JPRB,   0.8289941_JPRB,   0.0000059_JPRB,   0.0129137_JPRB,   0.0006305_JPRB,  &
&        1.8178842_JPRB,  -0.0024927_JPRB,   0.8428823_JPRB,   0.0000036_JPRB,   0.0130272_JPRB,   0.0006039_JPRB,  &
&        1.7702873_JPRB,  -0.0021033_JPRB,   0.8510188_JPRB,   0.0000028_JPRB,   0.0129360_JPRB,   0.0005815_JPRB,  &
&        1.6932452_JPRB,  -0.0014642_JPRB,   0.8651679_JPRB,   0.0000016_JPRB,   0.0126256_JPRB,   0.0005384_JPRB,  &
&        1.6589667_JPRB,  -0.0011792_JPRB,   0.8714384_JPRB,   0.0000010_JPRB,   0.0124445_JPRB,   0.0005185_JPRB,  &
&        1.6274499_JPRB,  -0.0009141_JPRB,   0.8772205_JPRB,   0.0000005_JPRB,   0.0122500_JPRB,   0.0004994_JPRB,  &
&        1.5591856_JPRB,  -0.0003493_JPRB,   0.8903558_JPRB,  -0.0000007_JPRB,   0.0119082_JPRB,   0.0004578_JPRB,  &
&        1.5281008_JPRB,  -0.0000845_JPRB,   0.8969700_JPRB,  -0.0000012_JPRB,   0.0116392_JPRB,   0.0004342_JPRB,  &
&        1.4720538_JPRB,   0.0004054_JPRB,   0.9102831_JPRB,  -0.0000023_JPRB,   0.0109944_JPRB,   0.0003828_JPRB,  &
&        1.4050366_JPRB,   0.0009696_JPRB,   0.9242925_JPRB,  -0.0000035_JPRB,   0.0103478_JPRB,   0.0003310_JPRB,  &
&        1.3317604_JPRB,   0.0015881_JPRB,   0.9414724_JPRB,  -0.0000048_JPRB,   0.0096191_JPRB,   0.0002671_JPRB,  &
&        1.2446173_JPRB,   0.0023148_JPRB,   0.9599643_JPRB,  -0.0000063_JPRB,   0.0087349_JPRB,   0.0001977_JPRB,  &
&        1.1405404_JPRB,   0.0031337_JPRB,   0.9806104_JPRB,  -0.0000080_JPRB,   0.0079057_JPRB,   0.0001251_JPRB,  &
&        0.8781747_JPRB,   0.0053313_JPRB,   1.0386268_JPRB,  -0.0000126_JPRB,   0.0053688_JPRB,  -0.0000909_JPRB,  &
&        0.6416426_JPRB,   0.0073584_JPRB,   1.0942345_JPRB,  -0.0000170_JPRB,   0.0025222_JPRB,  -0.0003097_JPRB,  &
&        0.6909781_JPRB,   0.0069657_JPRB,   1.0893868_JPRB,  -0.0000163_JPRB,   0.0019700_JPRB,  -0.0003113_JPRB,  &
&        1.2229234_JPRB,   0.0026996_JPRB,   1.0071839_JPRB,  -0.0000080_JPRB,   0.0023960_JPRB,  -0.0000951_JPRB,  &
&        1.4639688_JPRB,   0.0007803_JPRB,   0.9657423_JPRB,  -0.0000043_JPRB,   0.0028624_JPRB,   0.0000189_JPRB,  &
&        1.8573928_JPRB,  -0.0023649_JPRB,   0.8997036_JPRB,   0.0000018_JPRB,   0.0038948_JPRB,   0.0002038_JPRB,  &
&        2.0973597_JPRB,  -0.0042824_JPRB,   0.8572829_JPRB,   0.0000054_JPRB,   0.0047148_JPRB,   0.0003258_JPRB,  &
&        2.3809838_JPRB,  -0.0065378_JPRB,   0.8046702_JPRB,   0.0000098_JPRB,   0.0064647_JPRB,   0.0004927_JPRB,  &
&        2.4156959_JPRB,  -0.0067842_JPRB,   0.7983734_JPRB,   0.0000102_JPRB,   0.0067242_JPRB,   0.0005120_JPRB,  &
&        2.4322994_JPRB,  -0.0069753_JPRB,   0.7847086_JPRB,   0.0000108_JPRB,   0.0083642_JPRB,   0.0005890_JPRB,  &
&        2.4262083_JPRB,  -0.0070206_JPRB,   0.7764770_JPRB,   0.0000111_JPRB,   0.0095590_JPRB,   0.0006444_JPRB,  &
&        2.4234598_JPRB,  -0.0070647_JPRB,   0.7715409_JPRB,   0.0000113_JPRB,   0.0102539_JPRB,   0.0006774_JPRB,  &
&        2.4189966_JPRB,  -0.0072104_JPRB,   0.7602580_JPRB,   0.0000120_JPRB,   0.0118468_JPRB,   0.0007549_JPRB,  &
&        2.3874373_JPRB,  -0.0070481_JPRB,   0.7559499_JPRB,   0.0000119_JPRB,   0.0130215_JPRB,   0.0008002_JPRB,  &
&        2.3018670_JPRB,  -0.0064634_JPRB,   0.7583905_JPRB,   0.0000111_JPRB,   0.0144409_JPRB,   0.0008372_JPRB,  &
&        2.2505004_JPRB,  -0.0062093_JPRB,   0.7576658_JPRB,   0.0000110_JPRB,   0.0155949_JPRB,   0.0008794_JPRB,  &
&        2.2113109_JPRB,  -0.0059354_JPRB,   0.7596725_JPRB,   0.0000106_JPRB,   0.0161519_JPRB,   0.0008907_JPRB,  &
&        2.1508124_JPRB,  -0.0054787_JPRB,   0.7653400_JPRB,   0.0000098_JPRB,   0.0165641_JPRB,   0.0008889_JPRB,  &
&        2.1196668_JPRB,  -0.0052749_JPRB,   0.7671259_JPRB,   0.0000095_JPRB,   0.0170279_JPRB,   0.0008985_JPRB,  &
&        2.1034482_JPRB,  -0.0051556_JPRB,   0.7685494_JPRB,   0.0000093_JPRB,   0.0171184_JPRB,   0.0008983_JPRB,  &
&        2.0882680_JPRB,  -0.0050523_JPRB,   0.7698535_JPRB,   0.0000091_JPRB,   0.0172603_JPRB,   0.0008998_JPRB,  &
&        2.0696313_JPRB,  -0.0049064_JPRB,   0.7725833_JPRB,   0.0000089_JPRB,   0.0172670_JPRB,   0.0008936_JPRB,  &
&        2.0502210_JPRB,  -0.0047470_JPRB,   0.7767609_JPRB,   0.0000085_JPRB,   0.0170944_JPRB,   0.0008783_JPRB,  &
&        2.2100263_JPRB,  -0.0059945_JPRB,   0.7536364_JPRB,   0.0000108_JPRB,   0.0169827_JPRB,   0.0009291_JPRB,  &
&        2.2840369_JPRB,  -0.0065846_JPRB,   0.7423302_JPRB,   0.0000119_JPRB,   0.0170740_JPRB,   0.0009582_JPRB,  &
&        2.4274824_JPRB,  -0.0075782_JPRB,   0.7261659_JPRB,   0.0000136_JPRB,   0.0164940_JPRB,   0.0009731_JPRB,  &
&        2.5274956_JPRB,  -0.0081268_JPRB,   0.7212195_JPRB,   0.0000142_JPRB,   0.0154501_JPRB,   0.0009452_JPRB,  &
&        2.5251822_JPRB,  -0.0080522_JPRB,   0.7229795_JPRB,   0.0000140_JPRB,   0.0153491_JPRB,   0.0009357_JPRB,  &
&        2.4855759_JPRB,  -0.0076167_JPRB,   0.7314413_JPRB,   0.0000130_JPRB,   0.0152119_JPRB,   0.0009029_JPRB,  &
&        1.2105653_JPRB,   0.0055168_JPRB,   1.1423235_JPRB,  -0.0000191_JPRB,  -0.0141616_JPRB,  -0.0010229_JPRB,  &
&        1.0828085_JPRB,   0.0060361_JPRB,   1.1430210_JPRB,  -0.0000193_JPRB,  -0.0118738_JPRB,  -0.0009406_JPRB,  &
&        1.1349387_JPRB,   0.0057836_JPRB,   1.1409335_JPRB,  -0.0000191_JPRB,  -0.0126311_JPRB,  -0.0009646_JPRB,  &
&        1.2128894_JPRB,   0.0055563_JPRB,   1.1444103_JPRB,  -0.0000193_JPRB,  -0.0144012_JPRB,  -0.0010363_JPRB,  &
&        1.0387275_JPRB,   0.0063735_JPRB,   1.1490698_JPRB,  -0.0000198_JPRB,  -0.0116912_JPRB,  -0.0009463_JPRB,  &
&        0.7499074_JPRB,   0.0088119_JPRB,   1.1974345_JPRB,  -0.0000249_JPRB,  -0.0133488_JPRB,  -0.0011151_JPRB,  &
&        0.7838918_JPRB,   0.0085241_JPRB,   1.1986148_JPRB,  -0.0000241_JPRB,  -0.0138819_JPRB,  -0.0011151_JPRB,  &
&        0.8011539_JPRB,   0.0084193_JPRB,   1.1992733_JPRB,  -0.0000237_JPRB,  -0.0140699_JPRB,  -0.0011152_JPRB,  &
&        0.8215273_JPRB,   0.0084761_JPRB,   1.1974370_JPRB,  -0.0000238_JPRB,  -0.0140513_JPRB,  -0.0011075_JPRB/),&
&     SHAPE =(/ n_baran2013_sca, n_baran2013_wn/))

  REAL(KIND=JPRB), PUBLIC, PARAMETER :: baran2013_regcoef_bpr(n_baran2013_bpr, n_baran2013_wn) = &  
  RESHAPE( (/ &
&        0.1851917_JPRB,     -0.0006491_JPRB,     -0.0244456_JPRB,  &
&        0.1731565_JPRB,     -0.0006007_JPRB,     -0.0231140_JPRB,  &
&        0.1528599_JPRB,     -0.0005279_JPRB,     -0.0224386_JPRB,  &
&        0.1496294_JPRB,     -0.0005134_JPRB,     -0.0219574_JPRB,  &
&        0.1449891_JPRB,     -0.0004940_JPRB,     -0.0215721_JPRB,  &
&        0.1389072_JPRB,     -0.0004702_JPRB,     -0.0210243_JPRB,  &
&        0.1288915_JPRB,     -0.0004157_JPRB,     -0.0195136_JPRB,  &
&        0.1191244_JPRB,     -0.0003664_JPRB,     -0.0179957_JPRB,  &
&        0.1148952_JPRB,     -0.0003449_JPRB,     -0.0172313_JPRB,  &
&        0.1085006_JPRB,     -0.0003126_JPRB,     -0.0159851_JPRB,  &
&        0.1058559_JPRB,     -0.0002995_JPRB,     -0.0154507_JPRB,  &
&        0.1036747_JPRB,     -0.0002885_JPRB,     -0.0149828_JPRB,  &
&        0.1000297_JPRB,     -0.0002715_JPRB,     -0.0142186_JPRB,  &
&        0.0980345_JPRB,     -0.0002631_JPRB,     -0.0138334_JPRB,  &
&        0.0938417_JPRB,     -0.0002471_JPRB,     -0.0130892_JPRB,  &
&        0.0897155_JPRB,     -0.0002354_JPRB,     -0.0124969_JPRB,  &
&        0.0852887_JPRB,     -0.0002243_JPRB,     -0.0119063_JPRB,  &
&        0.0805446_JPRB,     -0.0002143_JPRB,     -0.0113254_JPRB,  &
&        0.0761359_JPRB,     -0.0002030_JPRB,     -0.0105758_JPRB,  &
&        0.0670280_JPRB,     -0.0001832_JPRB,     -0.0093224_JPRB,  &
&        0.0591270_JPRB,     -0.0001717_JPRB,     -0.0086362_JPRB,  &
&        0.0520255_JPRB,     -0.0001723_JPRB,     -0.0090047_JPRB,  &
&        0.0564121_JPRB,     -0.0001980_JPRB,     -0.0094871_JPRB,  &
&        0.0595459_JPRB,     -0.0002092_JPRB,     -0.0095958_JPRB,  &
&        0.0672062_JPRB,     -0.0002320_JPRB,     -0.0097933_JPRB,  &
&        0.0731099_JPRB,     -0.0002473_JPRB,     -0.0099735_JPRB,  &
&        0.0854102_JPRB,     -0.0002860_JPRB,     -0.0109790_JPRB,  &
&        0.0881551_JPRB,     -0.0002958_JPRB,     -0.0113260_JPRB,  &
&        0.0947020_JPRB,     -0.0003221_JPRB,     -0.0122727_JPRB,  &
&        0.0983052_JPRB,     -0.0003374_JPRB,     -0.0128156_JPRB,  &
&        0.0998681_JPRB,     -0.0003450_JPRB,     -0.0131324_JPRB,  &
&        0.1025055_JPRB,     -0.0003561_JPRB,     -0.0135363_JPRB,  &
&        0.1024091_JPRB,     -0.0003580_JPRB,     -0.0137690_JPRB,  &
&        0.0992052_JPRB,     -0.0003489_JPRB,     -0.0137933_JPRB,  &
&        0.0958166_JPRB,     -0.0003393_JPRB,     -0.0138034_JPRB,  &
&        0.0918819_JPRB,     -0.0003235_JPRB,     -0.0133642_JPRB,  &
&        0.0853509_JPRB,     -0.0002966_JPRB,     -0.0125757_JPRB,  &
&        0.0813083_JPRB,     -0.0002797_JPRB,     -0.0120314_JPRB,  &
&        0.0793876_JPRB,     -0.0002718_JPRB,     -0.0117807_JPRB,  &
&        0.0777021_JPRB,     -0.0002645_JPRB,     -0.0115190_JPRB,  &
&        0.0748741_JPRB,     -0.0002535_JPRB,     -0.0111925_JPRB,  &
&        0.0703067_JPRB,     -0.0002360_JPRB,     -0.0104636_JPRB,  &
&        0.0821496_JPRB,     -0.0002880_JPRB,     -0.0116964_JPRB,  &
&        0.0900865_JPRB,     -0.0003194_JPRB,     -0.0125076_JPRB,  &
&        0.1103655_JPRB,     -0.0003941_JPRB,     -0.0144732_JPRB,  &
&        0.1316836_JPRB,     -0.0004620_JPRB,     -0.0162807_JPRB,  &
&        0.1343305_JPRB,     -0.0004717_JPRB,     -0.0166932_JPRB,  &
&        0.1392748_JPRB,     -0.0004981_JPRB,     -0.0181562_JPRB,  &
&        0.1414660_JPRB,     -0.0004842_JPRB,     -0.0202197_JPRB,  &
&        0.1022973_JPRB,     -0.0003502_JPRB,     -0.0163840_JPRB,  &
&        0.1143898_JPRB,     -0.0003947_JPRB,     -0.0176106_JPRB,  &
&        0.1486137_JPRB,     -0.0005018_JPRB,     -0.0205480_JPRB,  &
&        0.1046780_JPRB,     -0.0003332_JPRB,     -0.0159917_JPRB,  &
&        0.0788520_JPRB,     -0.0001640_JPRB,     -0.0088560_JPRB,  &
&        0.0848020_JPRB,     -0.0001427_JPRB,     -0.0079221_JPRB,  &
&        0.0925953_JPRB,     -0.0001414_JPRB,     -0.0078181_JPRB,  &
&        0.0993293_JPRB,     -0.0001208_JPRB,     -0.0068834_JPRB/),&
&     SHAPE =(/ n_baran2013_bpr, n_baran2013_wn/))

  REAL(KIND=JPRB), PUBLIC, PARAMETER :: baran2013_corcoef_bpr(n_baran2013_corbpr, n_baran2013_wn) =  &
  RESHAPE( (/ &
&      0.0540_JPRB,   -0.0993_JPRB,    2.8402_JPRB,    0.0900_JPRB,    0.0024_JPRB,    0.9682_JPRB,  &
&      0.0540_JPRB,   -0.1028_JPRB,    2.9408_JPRB,    0.0900_JPRB,    0.0033_JPRB,    0.9593_JPRB,  &
&      0.0540_JPRB,   -0.1111_JPRB,    3.0522_JPRB,    0.0900_JPRB,    0.0091_JPRB,    0.8976_JPRB,  &
&      0.0540_JPRB,   -0.1136_JPRB,    3.1053_JPRB,    0.0900_JPRB,    0.0087_JPRB,    0.8994_JPRB,  &
&      0.0540_JPRB,   -0.1263_JPRB,    3.3444_JPRB,    0.0900_JPRB,    0.0102_JPRB,    0.8829_JPRB,  &
&      0.0540_JPRB,   -0.1264_JPRB,    3.3623_JPRB,    0.0900_JPRB,    0.0126_JPRB,    0.8575_JPRB,  &
&      0.0560_JPRB,   -0.1378_JPRB,    3.4754_JPRB,    0.0900_JPRB,    0.0194_JPRB,    0.7898_JPRB,  &
&      0.0560_JPRB,   -0.1489_JPRB,    3.6417_JPRB,    0.0900_JPRB,    0.0259_JPRB,    0.7196_JPRB,  &
&      0.0560_JPRB,   -0.1514_JPRB,    3.6790_JPRB,    0.0900_JPRB,    0.0287_JPRB,    0.6879_JPRB,  &
&      0.0560_JPRB,   -0.1502_JPRB,    3.6586_JPRB,    0.0890_JPRB,    0.0324_JPRB,    0.6421_JPRB,  &
&      0.0560_JPRB,   -0.1489_JPRB,    3.6418_JPRB,    0.0880_JPRB,    0.0336_JPRB,    0.6250_JPRB,  &
&      0.0560_JPRB,   -0.1470_JPRB,    3.6114_JPRB,    0.0880_JPRB,    0.0349_JPRB,    0.6078_JPRB,  &
&      0.0550_JPRB,   -0.1504_JPRB,    3.7034_JPRB,    0.0870_JPRB,    0.0367_JPRB,    0.5811_JPRB,  &
&      0.0550_JPRB,   -0.1443_JPRB,    3.6134_JPRB,    0.0850_JPRB,    0.0362_JPRB,    0.5789_JPRB,  &
&      0.0540_JPRB,   -0.1393_JPRB,    3.5836_JPRB,    0.0830_JPRB,    0.0369_JPRB,    0.5579_JPRB,  &
&      0.0510_JPRB,   -0.1383_JPRB,    3.6760_JPRB,    0.0790_JPRB,    0.0352_JPRB,    0.5591_JPRB,  &
&      0.0490_JPRB,   -0.1255_JPRB,    3.5541_JPRB,    0.0750_JPRB,    0.0336_JPRB,    0.5575_JPRB,  &
&      0.0460_JPRB,   -0.1116_JPRB,    3.4251_JPRB,    0.0710_JPRB,    0.0319_JPRB,    0.5544_JPRB,  &
&      0.0430_JPRB,   -0.0936_JPRB,    3.1644_JPRB,    0.0660_JPRB,    0.0292_JPRB,    0.5650_JPRB,  &
&      0.0370_JPRB,   -0.0610_JPRB,    2.6363_JPRB,    0.0580_JPRB,    0.0252_JPRB,    0.5694_JPRB,  &
&      0.0310_JPRB,   -0.0453_JPRB,    2.4477_JPRB,    0.0500_JPRB,    0.0209_JPRB,    0.5866_JPRB,  &
&      0.0240_JPRB,   -0.0487_JPRB,    3.0056_JPRB,    0.0430_JPRB,    0.0170_JPRB,    0.6072_JPRB,  &
&      0.0230_JPRB,   -0.0420_JPRB,    2.8717_JPRB,    0.0430_JPRB,    0.0126_JPRB,    0.7048_JPRB,  &
&      0.0230_JPRB,   -0.0406_JPRB,    2.7982_JPRB,    0.0430_JPRB,    0.0103_JPRB,    0.7588_JPRB,  &
&      0.0230_JPRB,   -0.0373_JPRB,    2.5561_JPRB,    0.0430_JPRB,    0.0051_JPRB,    0.8780_JPRB,  &
&      0.0250_JPRB,   -0.0346_JPRB,    2.3391_JPRB,    0.0430_JPRB,    0.0027_JPRB,    0.9329_JPRB,  &
&      0.0290_JPRB,   -0.0361_JPRB,    2.2471_JPRB,    0.0450_JPRB,    0.0020_JPRB,    0.9533_JPRB,  &
&      0.0300_JPRB,   -0.0373_JPRB,    2.2566_JPRB,    0.0470_JPRB,    0.0014_JPRB,    0.9643_JPRB,  &
&      0.0310_JPRB,   -0.0428_JPRB,    2.3961_JPRB,    0.0500_JPRB,    0.0006_JPRB,    0.9769_JPRB,  &
&      0.0310_JPRB,   -0.0477_JPRB,    2.5434_JPRB,    0.0500_JPRB,    0.0016_JPRB,    0.9623_JPRB,  &
&      0.0310_JPRB,   -0.0515_JPRB,    2.6647_JPRB,    0.0500_JPRB,    0.0021_JPRB,    0.9550_JPRB,  &
&      0.0310_JPRB,   -0.0573_JPRB,    2.8420_JPRB,    0.0500_JPRB,    0.0029_JPRB,    0.9449_JPRB,  &
&      0.0310_JPRB,   -0.0627_JPRB,    3.0173_JPRB,    0.0500_JPRB,    0.0035_JPRB,    0.9354_JPRB,  &
&      0.0310_JPRB,   -0.0664_JPRB,    3.1498_JPRB,    0.0500_JPRB,    0.0043_JPRB,    0.9210_JPRB,  &
&      0.0290_JPRB,   -0.0821_JPRB,    3.7364_JPRB,    0.0490_JPRB,    0.0057_JPRB,    0.8978_JPRB,  &
&      0.0290_JPRB,   -0.0807_JPRB,    3.7023_JPRB,    0.0490_JPRB,    0.0057_JPRB,    0.8936_JPRB,  &
&      0.0290_JPRB,   -0.0786_JPRB,    3.6744_JPRB,    0.0490_JPRB,    0.0056_JPRB,    0.8885_JPRB,  &
&      0.0290_JPRB,   -0.0765_JPRB,    3.6380_JPRB,    0.0490_JPRB,    0.0054_JPRB,    0.8880_JPRB,  &
&      0.0290_JPRB,   -0.0751_JPRB,    3.6041_JPRB,    0.0490_JPRB,    0.0054_JPRB,    0.8862_JPRB,  &
&      0.0280_JPRB,   -0.0784_JPRB,    3.7487_JPRB,    0.0480_JPRB,    0.0060_JPRB,    0.8744_JPRB,  &
&      0.0280_JPRB,   -0.0781_JPRB,    3.7718_JPRB,    0.0470_JPRB,    0.0067_JPRB,    0.8592_JPRB,  &
&      0.0270_JPRB,   -0.0727_JPRB,    3.6923_JPRB,    0.0450_JPRB,    0.0064_JPRB,    0.8571_JPRB,  &
&      0.0270_JPRB,   -0.0626_JPRB,    3.3482_JPRB,    0.0710_JPRB,   -0.0086_JPRB,    1.1248_JPRB,  &
&      0.0270_JPRB,   -0.0650_JPRB,    3.4017_JPRB,    0.0710_JPRB,   -0.0256_JPRB,    1.3598_JPRB,  &
&      0.0300_JPRB,   -0.0632_JPRB,    3.1144_JPRB,    0.0800_JPRB,   -0.0752_JPRB,    1.9265_JPRB,  &
&      0.0360_JPRB,   -0.0565_JPRB,    2.5842_JPRB,    0.0950_JPRB,   -0.1698_JPRB,    2.7973_JPRB,  &
&      0.0370_JPRB,   -0.0571_JPRB,    2.5608_JPRB,    0.0980_JPRB,   -0.1996_JPRB,    3.0591_JPRB,  &
&      0.0380_JPRB,   -0.0651_JPRB,    2.7259_JPRB,    0.1030_JPRB,   -0.2373_JPRB,    3.3012_JPRB,  &
&      0.0500_JPRB,   -0.0396_JPRB,    1.7584_JPRB,    0.1150_JPRB,    0.0679_JPRB,    0.4239_JPRB,  &
&      0.0420_JPRB,   -0.0449_JPRB,    2.0559_JPRB,    0.0900_JPRB,    0.0628_JPRB,    0.3139_JPRB,  &
&      0.0420_JPRB,   -0.0454_JPRB,    2.0265_JPRB,    0.1000_JPRB,    0.0690_JPRB,    0.3167_JPRB,  &
&      0.0550_JPRB,   -0.0386_JPRB,    1.6980_JPRB,    0.1300_JPRB,    0.0927_JPRB,    0.2896_JPRB,  &
&      0.0460_JPRB,   -0.0624_JPRB,    2.3007_JPRB,    0.0990_JPRB,    0.0756_JPRB,    0.2358_JPRB,  &
&      0.0530_JPRB,   -0.0617_JPRB,    2.1438_JPRB,    0.0800_JPRB,    0.0695_JPRB,    0.1334_JPRB,  &
&      0.0610_JPRB,   -0.0730_JPRB,    2.1420_JPRB,    0.0870_JPRB,    0.0762_JPRB,    0.1253_JPRB,  &
&      0.0700_JPRB,   -0.0911_JPRB,    2.2729_JPRB,    0.0920_JPRB,    0.0769_JPRB,    0.1750_JPRB,  &
&      0.0800_JPRB,   -0.1199_JPRB,    2.4696_JPRB,    0.1010_JPRB,    0.0866_JPRB,    0.1470_JPRB/),&
&     SHAPE =(/ n_baran2013_corbpr, n_baran2013_wn/))

  REAL(KIND=JPRB), PUBLIC, PARAMETER :: baran2013_regcoef_asym(n_baran2013_asym, n_baran2013_wn) = &
  RESHAPE( (/ &
&        0.6901861_JPRB,      0.0011314_JPRB,      0.0420089_JPRB,  &
&        0.7108122_JPRB,      0.0010497_JPRB,      0.0398526_JPRB,  &
&        0.7460123_JPRB,      0.0009251_JPRB,      0.0388418_JPRB,  &
&        0.7516418_JPRB,      0.0009002_JPRB,      0.0380475_JPRB,  &
&        0.7597262_JPRB,      0.0008667_JPRB,      0.0374155_JPRB,  &
&        0.7703859_JPRB,      0.0008254_JPRB,      0.0365203_JPRB,  &
&        0.7877809_JPRB,      0.0007312_JPRB,      0.0339506_JPRB,  &
&        0.8048611_JPRB,      0.0006457_JPRB,      0.0313818_JPRB,  &
&        0.8122979_JPRB,      0.0006082_JPRB,      0.0300870_JPRB,  &
&        0.8236014_JPRB,      0.0005518_JPRB,      0.0279738_JPRB,  &
&        0.8283160_JPRB,      0.0005288_JPRB,      0.0270672_JPRB,  &
&        0.8321937_JPRB,      0.0005094_JPRB,      0.0262702_JPRB,  &
&        0.8387337_JPRB,      0.0004793_JPRB,      0.0249700_JPRB,  &
&        0.8423452_JPRB,      0.0004643_JPRB,      0.0243145_JPRB,  &
&        0.8499326_JPRB,      0.0004354_JPRB,      0.0230401_JPRB,  &
&        0.8574321_JPRB,      0.0004138_JPRB,      0.0220141_JPRB,  &
&        0.8654879_JPRB,      0.0003929_JPRB,      0.0209721_JPRB,  &
&        0.8741516_JPRB,      0.0003733_JPRB,      0.0199178_JPRB,  &
&        0.8821572_JPRB,      0.0003516_JPRB,      0.0185424_JPRB,  &
&        0.8986307_JPRB,      0.0003117_JPRB,      0.0161354_JPRB,  &
&        0.9130764_JPRB,      0.0002842_JPRB,      0.0146614_JPRB,  &
&        0.9263324_JPRB,      0.0002738_JPRB,      0.0148818_JPRB,  &
&        0.9197261_JPRB,      0.0003123_JPRB,      0.0156101_JPRB,  &
&        0.9145397_JPRB,      0.0003314_JPRB,      0.0158369_JPRB,  &
&        0.9013030_JPRB,      0.0003737_JPRB,      0.0163492_JPRB,  &
&        0.8907272_JPRB,      0.0004045_JPRB,      0.0168210_JPRB,  &
&        0.8686420_JPRB,      0.0004788_JPRB,      0.0188405_JPRB,  &
&        0.8637024_JPRB,      0.0004973_JPRB,      0.0195080_JPRB,  &
&        0.8519753_JPRB,      0.0005459_JPRB,      0.0212753_JPRB,  &
&        0.8455184_JPRB,      0.0005738_JPRB,      0.0222700_JPRB,  &
&        0.8427179_JPRB,      0.0005875_JPRB,      0.0228379_JPRB,  &
&        0.8379359_JPRB,      0.0006080_JPRB,      0.0235713_JPRB,  &
&        0.8381014_JPRB,      0.0006112_JPRB,      0.0239756_JPRB,  &
&        0.8438530_JPRB,      0.0005943_JPRB,      0.0239864_JPRB,  &
&        0.8499410_JPRB,      0.0005763_JPRB,      0.0239640_JPRB,  &
&        0.8569039_JPRB,      0.0005477_JPRB,      0.0231559_JPRB,  &
&        0.8684008_JPRB,      0.0004994_JPRB,      0.0217008_JPRB,  &
&        0.8755127_JPRB,      0.0004690_JPRB,      0.0206916_JPRB,  &
&        0.8788729_JPRB,      0.0004548_JPRB,      0.0202247_JPRB,  &
&        0.8818137_JPRB,      0.0004416_JPRB,      0.0197434_JPRB,  &
&        0.8867517_JPRB,      0.0004218_JPRB,      0.0191250_JPRB,  &
&        0.8948458_JPRB,      0.0003892_JPRB,      0.0177479_JPRB,  &
&        0.8747720_JPRB,      0.0004783_JPRB,      0.0199766_JPRB,  &
&        0.8608581_JPRB,      0.0005350_JPRB,      0.0215134_JPRB,  &
&        0.8243353_JPRB,      0.0006736_JPRB,      0.0252122_JPRB,  &
&        0.7851973_JPRB,      0.0008028_JPRB,      0.0285172_JPRB,  &
&        0.7803391_JPRB,      0.0008208_JPRB,      0.0292359_JPRB,  &
&        0.7714771_JPRB,      0.0008677_JPRB,      0.0317564_JPRB,  &
&        0.7659616_JPRB,      0.0008490_JPRB,      0.0350750_JPRB,  &
&        0.8365779_JPRB,      0.0006072_JPRB,      0.0286217_JPRB,  &
&        0.8148567_JPRB,      0.0006876_JPRB,      0.0307567_JPRB,  &
&        0.7532374_JPRB,      0.0008799_JPRB,      0.0355370_JPRB,  &
&        0.8313449_JPRB,      0.0005829_JPRB,      0.0279760_JPRB,  &
&        0.8767821_JPRB,      0.0002874_JPRB,      0.0156145_JPRB,  &
&        0.8648919_JPRB,      0.0002563_JPRB,      0.0139932_JPRB,  &
&        0.8518218_JPRB,      0.0002487_JPRB,      0.0136329_JPRB,  &
&        0.8402393_JPRB,      0.0002107_JPRB,      0.0119069_JPRB/),&
&     SHAPE =(/ n_baran2013_asym, n_baran2013_wn/))

  REAL(KIND=JPRB), PUBLIC, PARAMETER :: baran2013_corcoef_asym(n_baran2013_corasym, n_baran2013_wn) = &
  RESHAPE( (/ &
&      0.8580_JPRB,    0.0178_JPRB,    0.9808_JPRB,    0.9200_JPRB,   -1.7267_JPRB,    2.8752_JPRB,  &
&      0.8580_JPRB,    0.0239_JPRB,    0.9736_JPRB,    0.9200_JPRB,   -1.8368_JPRB,    2.9913_JPRB,  &
&      0.8580_JPRB,    0.0752_JPRB,    0.9133_JPRB,    0.9200_JPRB,   -1.9223_JPRB,    3.0889_JPRB,  &
&      0.8580_JPRB,    0.0740_JPRB,    0.9151_JPRB,    0.9200_JPRB,   -1.9836_JPRB,    3.1548_JPRB,  &
&      0.8580_JPRB,    0.0877_JPRB,    0.8990_JPRB,    0.9200_JPRB,   -2.1995_JPRB,    3.3892_JPRB,  &
&      0.8580_JPRB,    0.1080_JPRB,    0.8751_JPRB,    0.9200_JPRB,   -2.2208_JPRB,    3.4107_JPRB,  &
&      0.8590_JPRB,    0.1654_JPRB,    0.8067_JPRB,    0.9170_JPRB,   -2.3639_JPRB,    3.5763_JPRB,  &
&      0.8590_JPRB,    0.2226_JPRB,    0.7395_JPRB,    0.9170_JPRB,   -2.5076_JPRB,    3.7367_JPRB,  &
&      0.8590_JPRB,    0.2494_JPRB,    0.7083_JPRB,    0.9170_JPRB,   -2.5432_JPRB,    3.7763_JPRB,  &
&      0.8600_JPRB,    0.2898_JPRB,    0.6619_JPRB,    0.9170_JPRB,   -2.5446_JPRB,    3.7777_JPRB,  &
&      0.8610_JPRB,    0.3057_JPRB,    0.6439_JPRB,    0.9170_JPRB,   -2.5194_JPRB,    3.7497_JPRB,  &
&      0.8610_JPRB,    0.3214_JPRB,    0.6261_JPRB,    0.9170_JPRB,   -2.4934_JPRB,    3.7209_JPRB,  &
&      0.8670_JPRB,    0.3320_JPRB,    0.6156_JPRB,    0.9190_JPRB,   -2.5987_JPRB,    3.8323_JPRB,  &
&      0.8700_JPRB,    0.3373_JPRB,    0.6106_JPRB,    0.9190_JPRB,   -2.5134_JPRB,    3.7373_JPRB,  &
&      0.8720_JPRB,    0.3609_JPRB,    0.5852_JPRB,    0.9200_JPRB,   -2.4163_JPRB,    3.6255_JPRB,  &
&      0.8760_JPRB,    0.3776_JPRB,    0.5685_JPRB,    0.9240_JPRB,   -2.4334_JPRB,    3.6334_JPRB,  &
&      0.8840_JPRB,    0.3799_JPRB,    0.5694_JPRB,    0.9290_JPRB,   -2.4357_JPRB,    3.6231_JPRB,  &
&      0.8900_JPRB,    0.3946_JPRB,    0.5562_JPRB,    0.9340_JPRB,   -2.3232_JPRB,    3.4876_JPRB,  &
&      0.9000_JPRB,    0.3872_JPRB,    0.5688_JPRB,    0.9390_JPRB,   -2.1042_JPRB,    3.2416_JPRB,  &
&      0.9140_JPRB,    0.4051_JPRB,    0.5560_JPRB,    0.9490_JPRB,   -1.6568_JPRB,    2.7464_JPRB,  &
&      0.9260_JPRB,    0.4149_JPRB,    0.5517_JPRB,    0.9590_JPRB,   -1.5492_JPRB,    2.6161_JPRB,  &
&      0.9370_JPRB,    0.4099_JPRB,    0.5627_JPRB,    0.9700_JPRB,   -2.2115_JPRB,    3.2814_JPRB,  &
&      0.9370_JPRB,    0.3240_JPRB,    0.6547_JPRB,    0.9720_JPRB,   -2.1537_JPRB,    3.2156_JPRB,  &
&      0.9370_JPRB,    0.2707_JPRB,    0.7113_JPRB,    0.9720_JPRB,   -2.0765_JPRB,    3.1366_JPRB,  &
&      0.9330_JPRB,    0.1694_JPRB,    0.8189_JPRB,    0.9680_JPRB,   -1.6482_JPRB,    2.7003_JPRB,  &
&      0.9330_JPRB,    0.0877_JPRB,    0.9067_JPRB,    0.9660_JPRB,   -1.4782_JPRB,    2.5292_JPRB,  &
&      0.9320_JPRB,    0.0425_JPRB,    0.9549_JPRB,    0.9610_JPRB,   -1.3426_JPRB,    2.3961_JPRB,  &
&      0.9300_JPRB,    0.0415_JPRB,    0.9559_JPRB,    0.9600_JPRB,   -1.3694_JPRB,    2.4255_JPRB,  &
&      0.9270_JPRB,    0.0454_JPRB,    0.9515_JPRB,    0.9580_JPRB,   -1.4700_JPRB,    2.5328_JPRB,  &
&      0.9270_JPRB,    0.0552_JPRB,    0.9403_JPRB,    0.9580_JPRB,   -1.6109_JPRB,    2.6805_JPRB,  &
&      0.9270_JPRB,    0.0614_JPRB,    0.9334_JPRB,    0.9580_JPRB,   -1.7335_JPRB,    2.8085_JPRB,  &
&      0.9270_JPRB,    0.0686_JPRB,    0.9251_JPRB,    0.9580_JPRB,   -1.9014_JPRB,    2.9843_JPRB,  &
&      0.9270_JPRB,    0.0770_JPRB,    0.9159_JPRB,    0.9580_JPRB,   -2.0599_JPRB,    3.1496_JPRB,  &
&      0.9270_JPRB,    0.0906_JPRB,    0.9011_JPRB,    0.9580_JPRB,   -2.1919_JPRB,    3.2863_JPRB,  &
&      0.9270_JPRB,    0.1057_JPRB,    0.8846_JPRB,    0.9620_JPRB,   -2.7902_JPRB,    3.9060_JPRB,  &
&      0.9270_JPRB,    0.1090_JPRB,    0.8814_JPRB,    0.9620_JPRB,   -2.8472_JPRB,    3.9640_JPRB,  &
&      0.9270_JPRB,    0.1174_JPRB,    0.8730_JPRB,    0.9620_JPRB,   -2.8158_JPRB,    3.9290_JPRB,  &
&      0.9270_JPRB,    0.1201_JPRB,    0.8706_JPRB,    0.9620_JPRB,   -2.7487_JPRB,    3.8576_JPRB,  &
&      0.9270_JPRB,    0.1226_JPRB,    0.8682_JPRB,    0.9620_JPRB,   -2.7299_JPRB,    3.8372_JPRB,  &
&      0.9300_JPRB,    0.1406_JPRB,    0.8485_JPRB,    0.9630_JPRB,   -2.7983_JPRB,    3.9071_JPRB,  &
&      0.9320_JPRB,    0.1567_JPRB,    0.8312_JPRB,    0.9630_JPRB,   -2.7936_JPRB,    3.9005_JPRB,  &
&      0.9350_JPRB,    0.1596_JPRB,    0.8290_JPRB,    0.9650_JPRB,   -2.7973_JPRB,    3.8986_JPRB,  &
&      0.8900_JPRB,   -0.0727_JPRB,    1.0823_JPRB,    0.9650_JPRB,   -2.5261_JPRB,    3.6162_JPRB,  &
&      0.8900_JPRB,   -0.2752_JPRB,    1.3101_JPRB,    0.9650_JPRB,   -2.5325_JPRB,    3.6250_JPRB,  &
&      0.8700_JPRB,   -0.7691_JPRB,    1.8819_JPRB,    0.9600_JPRB,   -2.2651_JPRB,    3.3586_JPRB,  &
&      0.8470_JPRB,   -1.5745_JPRB,    2.8549_JPRB,    0.9500_JPRB,   -1.6945_JPRB,    2.7814_JPRB,  &
&      0.8420_JPRB,   -1.7870_JPRB,    3.1157_JPRB,    0.9480_JPRB,   -1.6417_JPRB,    2.7288_JPRB,  &
&      0.8340_JPRB,   -1.9912_JPRB,    3.3875_JPRB,    0.9470_JPRB,   -1.7988_JPRB,    2.8977_JPRB,  &
&      0.8100_JPRB,    0.4418_JPRB,    0.4528_JPRB,    0.9200_JPRB,   -0.6740_JPRB,    1.7306_JPRB,  &
&      0.8550_JPRB,    0.5821_JPRB,    0.3181_JPRB,    0.9400_JPRB,   -1.0642_JPRB,    2.1322_JPRB,  &
&      0.8400_JPRB,    0.5443_JPRB,    0.3515_JPRB,    0.9350_JPRB,   -0.9241_JPRB,    1.9875_JPRB,  &
&      0.7950_JPRB,    0.4511_JPRB,    0.4320_JPRB,    0.9200_JPRB,   -0.6842_JPRB,    1.7444_JPRB,  &
&      0.8500_JPRB,    0.5735_JPRB,    0.3235_JPRB,    0.9300_JPRB,   -1.1451_JPRB,    2.2317_JPRB,  &
&      0.8750_JPRB,    0.7473_JPRB,    0.1455_JPRB,    0.9200_JPRB,   -1.0659_JPRB,    2.1583_JPRB,  &
&      0.8650_JPRB,    0.7233_JPRB,    0.1624_JPRB,    0.9050_JPRB,   -1.0956_JPRB,    2.2127_JPRB,  &
&      0.8500_JPRB,    0.7193_JPRB,    0.1531_JPRB,    0.8900_JPRB,   -1.1074_JPRB,    2.2455_JPRB,  &
&      0.8390_JPRB,    0.6935_JPRB,    0.1723_JPRB,    0.8700_JPRB,   -1.1235_JPRB,    2.2898_JPRB/),&
&     SHAPE =(/ n_baran2013_corasym, n_baran2013_wn/))

! Baran database training limits
  REAL(KIND=JPRB), PUBLIC,    PARAMETER :: baran2013_iwc_min  = 6.0000001e-06_JPRB
  REAL(KIND=JPRB), PUBLIC,    PARAMETER :: baran2013_iwc_max  = 1.969466_JPRB
  REAL(KIND=JPRB), PUBLIC,    PARAMETER :: baran2013_temp_min = 193.157_JPRB
  REAL(KIND=JPRB), PUBLIC,    PARAMETER :: baran2013_temp_max = 273.12701_JPRB

END MODULE mod_rttov_baran2013_icldata
!
