PROGRAM US76_TO_HDF5
! Description:
! Creates a HDF5 file containing the 6 Standard Atmospheres
! extrated from LBLRTM v12.1 lblatm.f90 file
! This is strictly the same profiles as described in
!  AFGL atmospheric constituent profiles (0-120km)
!  by Anderson, G. P. 
!  Hanscom AFB, MA : Optical Physics Division, Air Force Geophysics Laboratory, [1986].
!  AFGL-TR ; 86-0110. . U.S. Air Force Geophysics Laboratory. Optical Physics Division.
!
! All profiles are located over Atlantic Ocean.
! Original latitude and month are kept.
!
! Copyright:
!    This software was developed within the context of
!    the EUMETSAT Satellite Application Facility on
!    Numerical Weather Prediction (NWP SAF), under the
!    Cooperation Agreement dated 25 November 1998, between
!    EUMETSAT and the Met Office, UK, by one or more partners
!    within the NWP SAF. The partners in the NWP SAF are
!    the Met Office, ECMWF, KNMI and MeteoFrance.
!
!    Copyright 2012, EUMETSAT, All Rights Reserved.

  USE PARKIND1, ONLY : JPIM
  USE RTTOV_TYPES, ONLY : RTTOV_OPTIONS, PROFILE_TYPE
  USE RTTOV_UNIX_ENV, ONLY : RTTOV_EXIT
#ifdef _RTTOV_HDF
  USE HDF5
  USE RTTOV_HDF_MOD
#endif

#include "throw.h"

  IMPLICIT NONE

#include "rttov_alloc_prof.interface"
#include "rttov_print_profile.interface"
#ifdef _RTTOV_HDF
#include "rttov_hdf_save.interface"
#endif
! RTTOV declarations
  INTEGER(KIND=JPIM) :: I
  TYPE(RTTOV_OPTIONS):: OPTS
  TYPE(PROFILE_TYPE) :: PROFILES(6)
  INTEGER(KIND=JPIM) :: NPROF, NLEVELS
  INTEGER(KIND=JPIM) :: ASW
  INTEGER(KIND=JPIM) :: ERR

TRY

#ifndef _RTTOV_HDF
  ERR=1_JPIM
  THROWM(ERR.NE.0,"Program should be compiled with HDF5 enabled")
#endif

  WRITE(*,*) "Enter number of levels (max is 50)"
  WRITE(*,*) "recommended value for RTTOV v11 is 43 (0.045hPa)"
  READ(*,*) NLEVELS

  OPTS % RT_IR % OZONE_DATA = .TRUE.
  OPTS % RT_IR % CO2_DATA   = .TRUE.
  OPTS % RT_IR % N2O_DATA   = .TRUE.
  OPTS % RT_IR % CH4_DATA   = .TRUE.
  OPTS % RT_IR % CO_DATA    = .TRUE.

  ASW     = 1_JPIM ! ALLOCATE
  NPROF   = 6_JPIM

  CALL RTTOV_ALLOC_PROF (ERR, NPROF, PROFILES, &
    & NLEVELS, OPTS, ASW,&
    & INIT=.TRUE.)
  THROWM( ERR .NE. 0 , "RTTOV_ALLOC_PROF FATAL ERROR")


  DO I = 1, NPROF
    CALL RTTOV_US76_PROF(ERR, I, NLEVELS, PROFILES(I))
    THROW(ERR.NE.0)
    CALL RTTOV_PRINT_PROFILE (PROFILES(I))
  END DO
  
#ifdef _RTTOV_HDF
  CALL OPEN_HDF( .FALSE., ERR ) ! 32 bits
  THROW(ERR.NE.0)

  CALL RTTOV_HDF_SAVE( ERR, "us_standard_76.H5", '/PROFILES', CREATE=.TRUE., &
    &  PROFILES = PROFILES)
   THROW(ERR.NE.0)

  CALL RTTOV_HDF_SAVE( ERR,  "us_standard_76.H5", '/OPTIONS', CREATE=.FALSE., &
    &  OPTIONS = OPTS)
  THROW(ERR.NE.0)

  CALL CLOSE_HDF( ERR )
  THROW(ERR.NE.0)
#endif

PCATCH
END PROGRAM
